""" Copyright start
  Copyright (C) 2024 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end """

import uuid
import datetime

from django.db import models
from django.contrib.postgres.fields import ArrayField
from django.core.validators import MinValueValidator, MaxValueValidator
from ..constants import TTL_WEEKS


class XFJobBase(models.Model):
    uuid = models.UUIDField(primary_key=True, default=uuid.uuid4, editable=False, max_length=150)

    file = models.BinaryField(blank=True, null=True)

    PERCENTAGE_VALIDATOR = [MinValueValidator(0), MaxValueValidator(100)]

    status = models.TextField(blank=True, null=True)
    errorMessage = models.TextField(blank=True, null=True, db_column='error_message')
    logMessages = ArrayField(models.TextField(blank=True), blank=True, null=True, db_column='log_messages', default=list)
    progressPercent = models.SmallIntegerField(default=0, validators=PERCENTAGE_VALIDATOR, db_column='progress_percent')

    class Meta:
        abstract = True


class XFImportJob(XFJobBase):
    currentlyImporting = models.CharField(max_length=100, blank=True, null=True, db_column='currently_importing')
    options = models.JSONField(blank=True, null=True)

    @classmethod
    def ttl_delete(cls, weeks=TTL_WEEKS):
        delete_time = datetime.datetime.now() - datetime.timedelta(weeks=weeks)
        cls.objects.filter(createDate__lt=delete_time).delete()

    class Meta:
        abstract = True


class XFExportJob(XFJobBase):
    currentlyExporting = models.CharField(max_length=100, blank=True, null=True, db_column='currently_exporting')
    selections = models.JSONField(blank=True, null=True)

    class Meta:
        abstract = True
