SENSITIVE_PATTERNS = [
    (r'secretKey=[^&\s]+', 'secretKey=*****'),
    (r'pass=[^&\s]+', 'pass=*****'),
    (r'([\'"]?Authorization[\'"]?\s*:\s*[\'"]?)(Bearer|Basic|AR-JWT|Token|ApiKey|Splunk|IBToken|OAuth|MC)?\s*([A-Za-z0-9+/=._\-]+)([\'"]?)', r'\1\2 *****\4'),
]

#
# NOT using LOGGING_FILTERS from xf-utils for live-sync. This is because live-sync is ASGI service
# and it need asgi_correlation_id. xf-utils uses django_guid for correlation_id. Please check live-sync service
# after you update LOGGING_FILTERS.
#

LOGGING_FILTERS = {
    "correlation_id": {
        "()": "django_guid.log_filters.CorrelationId"
    },
    "tenantid": {
        "()": "xfutils.logging_guid.logging_filters.TenantIDLoggingFilter",
    },
    "mask_sensitive_data": {
        "()": "xfutils.logging_guid.logging_filters.MaskSensitiveDataFilter",
        "sensitive_patterns": SENSITIVE_PATTERNS
    }
}

LOGGING_FILTERS_CELERY = {
    "correlation_id": {
        "()": "django_guid.log_filters.CorrelationId"
    },
    "tenantid": {
        "()": "xfutils.logging_guid.logging_filters.TenantIDLoggingFilter",
    },
    "taskid": {
        "()": "xfutils.logging_guid.logging_filters.TaskIDLoggingFilter",
    },
    "mask_sensitive_data": {
        "()": "xfutils.logging_guid.logging_filters.MaskSensitiveDataFilter",
        "sensitive_patterns": SENSITIVE_PATTERNS
    }
}

LOGGING_HANDLERS = {
    "console": {
        "class": "logging.StreamHandler",
        "formatter": "verbose",
        "level": "DEBUG",
        "filters": ["correlation_id", "tenantid", "mask_sensitive_data"],
    }
}

LOGGING_HANDLERS_CELERY = {
    "console": {
        "class": "logging.StreamHandler",
        "formatter": "verbose",
        "level": "DEBUG",
        "filters": ["correlation_id", "tenantid", "taskid", "mask_sensitive_data"],
    }
}

LOGGING_FORMATTERS =  {
    "verbose": {
        "format": "[%(asctime)s] [%(process)d] [%(levelname)s] [%(tenantid)s] [%(correlation_id)s] " +
                    "[%(name)s:%(lineno)s] %(message)s",
        "datefmt": "%d/%b/%Y %H:%M:%S %Z"
    },
    "simple": {
        "format": "%(levelname)s %(message)s"
    }
}

LOGGING_FORMATTERS_CELERY = {
    "verbose": {
        "format": "[%(asctime)s] [%(process)d] [%(levelname)s] [%(tenantid)s] [%(correlation_id)s] " +
                    "[%(name)s:%(lineno)s] [%(taskid)s] %(message)s",
        "datefmt": "%d/%b/%Y %H:%M:%S %Z"
    },
    "simple": {
        "format": "%(levelname)s %(message)s"
    }
}