""" Copyright start
  Copyright (C) 2008 - 2023 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end """

import logging
import json
import re
from .context import tenantid
from .context import taskid

class TenantIDLoggingFilter(logging.Filter):
    def __init__(self):
        super().__init__()

    def filter(self, record):
        record.tenantid = tenantid.get()
        return True

class TaskIDLoggingFilter(logging.Filter):
    def __init__(self):
        super().__init__()

    def filter(self, record):
        record.taskid = taskid.get()
        return True

class MaskSensitiveDataFilter(logging.Filter):
    def __init__(self, sensitive_patterns=None):
        super().__init__()
        self.sensitive_patterns = sensitive_patterns or []

    def filter(self, record):
        record.msg = self.mask_sensitive_data(record.msg)
        if record.args:
            record.args = self.mask_args(record.args)
        return True

    def mask_args(self, args):
        if isinstance(args, str):
            return self.mask_sensitive_data(args)
        elif isinstance(args, dict):
            return {k: self.mask_sensitive_data(v) if isinstance(v, str) else v for k, v in args.items()}
        elif isinstance(args, list):
            return [self.mask_sensitive_data(arg) if isinstance(arg, str) else arg for arg in args]
        elif isinstance(args, tuple):
            return tuple(self.mask_sensitive_data(arg) if isinstance(arg, str) else arg for arg in args)
        elif isinstance(args, (int, float, bool)):
            return args
        return args

    def mask_sensitive_data(self, data):
        if isinstance(data, str):
            return self._mask_patterns(data)
        elif isinstance(data, dict):
            data_str = json.dumps(data)
            data_str = self._mask_patterns(data_str)
            return json.loads(data_str)
        elif isinstance(data, list):
            return [self.mask_sensitive_data(item) for item in data]
        elif isinstance(data, tuple):
            return tuple(self.mask_sensitive_data(item) for item in data)
        return data

    def _mask_patterns(self, data):
        for pattern, replacement in self.sensitive_patterns:
            data = re.sub(pattern, replacement, data)
        return data