import logging
from .metrics_service import notify_metrics

logger = logging.getLogger(__name__)

def monitor_license(license_serializer_data, metrics_host='metrics:443'):
    """
    license_serializer_data
    {
        "type": "subscription",
        "tenantid": "1090529_prod_10001",
        "tenantuuid": "4655e737-9938-4473-8ec7-2655dc8c70da",
        "description": "Prod license for user Niraj Nandane",
        "serialNumber": "FSRCLDTM21000002",
        "info": {
            "is_expired": false,
            "expiry_date": "2025-07-16",
            "days_to_expire": 360,
            "is_expiring": false,
            "concurent_playbook_executions": 1,
            "playbooks_per_day": -1,
            "storage": 200,
            "polling_frequency": 10,
            "push_events": 20,
            "playbooks_per_min": -1,
            "concurrent_developer_user_logins": 2,
            "api_key_logins": 2,
            "data_retention_period": 15,
            "number_of_agents": 2,
            "premium_subscription": "Required",
            "exec_env": "pooled"
        }
    }
    """
    tenantid = license_serializer_data['tenantid']
    logger.info(f"Monitoring license for tenant {tenantid}")
    
    payload = {
        "metrics": [
            {
                "tenantid": tenantid,
                "tenantuuid": license_serializer_data['tenantuuid'],
                "service_name": "auth",
                "name": "tenantinfo",
                #
                # Current license sync (similar to FDN sync) is not available, hence make ttl as -1.
                # TODO Whenever license sync (FDN sync similar) comes, call monitor_license from it
                #
                "ttl": -1,
                "unit": "info",
                "publish": True,
                "value": license_serializer_data
            }
        ],
        "formats": {
            "tenantinfo": [
                "xf_dplane_tenant{tenantid=\"${tenantid}\", tenantuuid=\"${tenantuuid}\"} 1",
                "xf_dplane_tenant_license{tenantid=\"${tenantid}\", serial_number=\"${value.serialNumber}\", expiry_date=\"${value.info.expiry_date}\", data_retention_period=\"${value.info.data_retention_period}\", concurrent_developer_user_logins=\"${value.info.concurrent_developer_user_logins}\", api_key_logins=\"${value.info.api_key_logins}\", number_of_agents=\"${value.info.number_of_agents}\"} 1",
                "xf_dplane_tenant_days_to_expire{tenantid=\"${tenantid}\"} ${value.info.days_to_expire}",
                "xf_dplane_tenant_workflow_details{tenantid=\"${tenantid}\", concurrent_playbook_executions=\"${value.info.concurrent_playbook_executions}\", playbooks_per_day=\"${value.info.playbooks_per_day}\", playbooks_per_min=\"${value.info.playbooks_per_min}\", polling_frequency=\"${value.info.polling_frequency}\", push_events=\"${value.info.push_events}\", exec_env=\"${value.info.exec_env}\"} 1",
                "xf_dplane_tenant_storage_limit{tenantid=\"${tenantid}\"} ${value.info.storage}"
            ]
        }
    }
    notify_metrics(payload, metrics_host)