import logging
import urllib3
from xfutils.utils import api_call
from xfutils.service_token import prepare_service_token

urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)

logger = logging.getLogger(__name__)

def notify_metrics(payload, metrics_host='metrics:443'):
    headers = {
        'Authorization': 'Service-Token {0}'.format(prepare_service_token()),
        'Content-Type': 'application/json'
    }
    METRICS_API_URL = "https://{0}/api/metrics/v1/metrics/".format(metrics_host)
    try:
        api_call(METRICS_API_URL, method='POST', body=payload,  headers=headers, verify=False, timeout=10)
        return True
    except Exception as e:
        logger.error(f"Got error while notifying metrics service: {str(e)}")
    return False

def query_metrics(payload, metrics_host='metrics:443'):
    headers = {
        'Authorization': 'Service-Token {0}'.format(prepare_service_token()),
        'Content-Type': 'application/json'
    }
    METRICS_QUERY_URL = "https://{0}/api/metrics/v1/metrics/query/".format(metrics_host)
    try:
        return api_call(METRICS_QUERY_URL, method='POST', body=payload,  headers=headers, verify=False, timeout=10)
    except Exception as e:
        logger.error(f"Got error while querying metrics service: {str(e)}")
    return None

def query_storage_usage(tenantid, storage_limit_gib, metrics_host='metrics:443'):
    if storage_limit_gib == -1:
        # -1 is unlimited
        return {
            'current_usage': None,
            'limit': storage_limit_gib,
            'should_limit': False
        }
    payload = {
        "tenantids": [tenantid],
        "query": {
            "service_name": "*",
            "operation": "sum",
            "metric_name": "storage_usage_bytes"
        }
    }
    api_response = query_metrics(payload, metrics_host)
    if not api_response:
        return {
            'current_usage': None,
            'limit': storage_limit_gib,
            'should_limit': False
        }
    current_usage_bytes = api_response.get(tenantid)
    # Convert bytes to GiB
    current_usage = current_usage_bytes / (1024 ** 3)
    current_usage_gib = float(f"{current_usage:.3f}")
    return {
        'current_usage': current_usage_gib,
        'limit': storage_limit_gib,
        'should_limit': current_usage_gib >= storage_limit_gib
    }

def register_intra_service_http_request_count(service_name, service_uuid, cache_key, metrics_host='metrics:443'):
    payload = {
        "metrics": [{
                "tenantid": service_name,
                "tenantuuid": service_uuid,
                "service_name": service_name,
                "ttl": -1, # In secs
                "name": "http_request_count",
                "cache_key": cache_key,
                "unit": "count",
                "publish": True,
                "value": 0
            }
        ],
        "formats": {
            "http_request_count": [
                "xf_dplane_http_request_count{service=\"${service_name}\"} ${value}"
            ]
        }
    }
    logger.info(f"Registering http_request_count metric with {metrics_host}")
    return notify_metrics(payload, metrics_host)