""" Copyright start
  Copyright (C) 2008 - 2023 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end """

import logging
import time
import threading
from django.conf import settings
from .metrics_service import register_intra_service_http_request_count

logger = logging.getLogger(__name__)

class RequestCountMiddleware(object):
    sync_capable = True
    async_capable = False
    
    def initialize(self, service_context):
        service_name = service_context.service_config.get('application').get('service_name')
        metric_id = '{0}:http_request_count'.format(service_name)
        self.cache_key = service_context.get_cache_key(service_name, metric_id, True)
        self.service_context = service_context

    def __init__(self, get_response, service_context):
        self.get_response = get_response
        self.initialize(service_context)
        service_name = self.service_context.service_config.get('application').get('service_name')
        service_uuid = self.service_context.service_config.get('application').get('service_uuid')
        metrics_host = self.service_context.service_config.get('application', {}).get('metrics_host', 'metrics:443')
        # Start the register_intra_service_http_request_count in a separate thread
        thread = threading.Thread(
            target=self._register_metrics,
            args=(service_name, service_uuid, self.cache_key, metrics_host)
        )
        thread.start()

    def _register_metrics(self, service_name, service_uuid, cache_key, metrics_host):
        max_retries = 5
        attempts = 0
        success = False
        while (attempts < max_retries) and (not success):
            try:
                success = register_intra_service_http_request_count(service_name, service_uuid, cache_key, metrics_host)
                if success:
                    logger.info("Successfully registered http_request_count")
                else:
                    logger.error("Failed to register http_request_count, retrying...")
                attempts += 1
            except Exception as e:
                logger.error(f"Exception occurred while registering http_request_count: {e}")
                attempts += 1
            if not success:
                sleep_sec = 5
                logger.warning(f"Sleeping for {sleep_sec} secs")
                time.sleep(sleep_sec)
        if not success:
            logger.error(f"Failed to register http_request_count after {max_retries} attempts")

    def __call__(self, request):
        self.service_context.raw_cache_client.incr(self.cache_key)
        response = self.get_response(request)
        return response

def get_request_count_middleware(get_response):
    return RequestCountMiddleware(
        get_response,
        service_context=settings.service_context
    )