import re
import logging
from rest_framework import permissions
from xfutils.monitor.policy_utils import get_policy_error_message

logger = logging.getLogger(__name__)

class DefaultPermission(permissions.BasePermission):

    def __init__(self, whitelisted_paths=None):
        super().__init__()
        self.whitelisted_paths = whitelisted_paths if whitelisted_paths else []

    def __call__(self):
        return self

    def has_permission(self, request, view):
        current_url = f'{request.method}:{request.path}'
        if current_url in self.whitelisted_paths:
            return True
        
        role = request.role
        for entity, perms in role.items():
            for perm in perms:
                for path in perm.get('paths', []):
                    if re.match(path, current_url):
                        return True
        if hasattr(request, 'policy') and request.policy:
            self.message = get_policy_error_message(request.policy)
        return False