import time

from .utils import get_ratelimit_model, get_cache_client, split_rate

RateLimit = get_ratelimit_model()
cache_client = get_cache_client()


def is_ratelimited(request, metric, rate, value=0, increment_by=1, **kwargs):
    thresold_value, seconds = split_rate(rate)
    #
    # Take global tenant and metric specific lock before proceeding.
    # There could be race condition where same call is coming twice.
    #
    # 101210_prod_1
    tenantid = request.tenantid
    # Take lock on Redis
    with cache_client.lock(f'lock_ratelimit_{tenantid}_{metric}'):
        current_epoch = time.time()
        reset_time = current_epoch + seconds
        try:
            ratelimit = RateLimit.objects.get(metric=metric)
        except RateLimit.DoesNotExist:
            ratelimit = RateLimit.initialize_ratelimit(metric=metric,
                                                       rate=rate,
                                                       reset_time=reset_time,
                                                       last_update_time=current_epoch,
                                                       value=value,
                                                       **kwargs)
        if current_epoch > ratelimit.reset_time:
            ratelimit.reset_ratelimit(reset_time, rate=rate, value=value)
        current_usage = ratelimit.value + increment_by
        if current_usage > thresold_value:
            return True
        ratelimit.last_update_time = current_epoch
        #
        # TODO Check if in any condition current_usage goes below 0.
        # For eg. on user logout, we may need to decrease value.
        #
        ratelimit.value = current_usage
        ratelimit.save()
        # if current_usage == 5:
        #    RateLimit.refresh_ratelimit(metric, '100/h', **kwargs)
        return False
