import uuid
import time

from django.db import models
from abc import ABCMeta

from .utils import split_rate


class AbstractModelMeta(ABCMeta, type(models.Model)):
    pass


class AbstractRateLimit(models.Model, metaclass=AbstractModelMeta):
    uuid = models.UUIDField(primary_key=True, default=uuid.uuid4, editable=False)
    metric = models.CharField(max_length=255)
    #
    # rate can be in form of 1000/86400s or 1000/24h or 1000/1440m or 1000/d
    #
    rate = models.CharField(max_length=255)
    value = models.PositiveIntegerField()
    # reset_time will be stored in epoch
    reset_time = models.PositiveIntegerField()
    # last_update_time will be stored in epoch
    last_update_time = models.PositiveIntegerField()

    class Meta:
        abstract = True
    
    @classmethod
    def initialize_ratelimit(cls, value=0, **kwargs):
        return cls.objects.create(value=value, **kwargs)

    def reset_ratelimit(self, reset_time, rate, value=0):
        self.reset_time = reset_time
        self.rate = rate
        self.value = value
        self.save()
    
    @classmethod
    def refresh_ratelimit(cls, metric, new_rate, value=0, **kwargs):
        current_epoch = time.time()
        thresold_value, seconds = split_rate(new_rate)
        new_reset_time = current_epoch + seconds
        try:
            ratelimit = cls.objects.get(metric=metric)
            ratelimit.reset_time = new_reset_time
            ratelimit.rate = new_rate
            ratelimit.last_update_time = current_epoch
            ratelimit.value = 0
            ratelimit.save()
        except cls.DoesNotExist:
            ratelimit = cls.initialize_ratelimit(metric=metric,
                                                 rate=new_rate,
                                                 reset_time=new_reset_time,
                                                 last_update_time=current_epoch,
                                                 value=value,
                                                 **kwargs)
            return ratelimit
