import logging
import re

from django.apps import apps as django_apps
from django.conf import settings
from django.core.exceptions import ImproperlyConfigured
from django.core.cache import cache

logger = logging.getLogger(__name__)

PERIODS = {
    's': 1,
    'm': 60,
    'h': 60 * 60,
    'd': 24 * 60 * 60
}

# 5/s, 50/d, 100/5m, 100/300s
rate_re = re.compile(r'(\d+)/(\d*)([smhd])?')


def split_rate(rate):
    count, multi, period = rate_re.match(rate).groups()
    count = int(count)
    if not period:
        period = 's'
    seconds = PERIODS[period.lower()]
    if multi:
        seconds = seconds * int(multi)
    return count, seconds


def get_cache_client():
    return cache.client.get_client()


def get_ratelimit_model():
    try:
        return django_apps.get_model(settings.RATELIMIT_MODEL, require_ready=False)
    except ValueError:
        raise ImproperlyConfigured(
            "RATELIMIT_MODEL must be of the form 'app_label.model_name'"
        )
    except LookupError:
        raise ImproperlyConfigured(
            "RATELIMIT_MODEL refers to model '%s' that has not been installed"
            % settings.RATELIMIT_MODEL
        )
