""" Copyright start
  Copyright (C) 2008 - 2023 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end """
import json
import logging
from django_guid import get_guid
from kombu import Connection, Exchange, pools

import xfutils.constants as constants

logger = logging.getLogger(__name__)


class RabbitMQPublisher:
    def __init__(self):
        self.connections = pools.Connections(limit=1)
        self.producers = pools.Producers(limit=self.connections.limit)
        config_data_dict = constants.ALL_CONFIG
        self.connection = Connection(config_data_dict.get('rmq', 'rmq_host'),
                                     config_data_dict.get('rmq', 'rmq_user'),
                                     config_data_dict.get('rmq', 'rmq_password'),
                                     config_data_dict.get('rmq', 'rmq_vhost'),
                                     config_data_dict.get('rmq', 'rmq_port'))

    def publish(self, exchange_name, routing_key, body, exchange_type='topic'):
        try:
            body = json.loads(body)
        except Exception:
            pass
        headers = {
            'tenantuuid':  body.get('tenantuuid'),
            'correlation_id': get_guid()
        }
        exchange = Exchange(exchange_name, exchange_type=exchange_type)
        with self.producers[self.connection].acquire(block=True) as producer:
            producer.publish(body, exchange=exchange, routing_key=routing_key, headers=headers, serializer='json')


def _publish_rmq_message(exchange_name, routing_key, body):
    rmq = constants.RMQ_PUBLISHER
    # Initialize rmq object if it's not already active
    if not rmq:
        constants.RMQ_PUBLISHER = RabbitMQPublisher()
        rmq = constants.RMQ_PUBLISHER
    rmq.publish(exchange_name, routing_key, body)
