#!/bin/bash
# Copyright start
# Copyright (C) 2008 - 2024 Fortinet Inc.
# All rights reserved.
# FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
# Copyright end

product_version="1.0.0"
#product_version="@product_version@"
time_stamp=$(date +"%F"-"%s")

get_password() {
    local pgpw_1
    local pgpw_2

    read -s -p "Enter a password to be used for the Agent database: " pgpw_1
    printf "\n"
    if [[ -z $pgpw_1 ]]; then
        printf "Password string may not be empty. Please try again.\n"
        return
    fi
    if [[ ! $pgpw_1 =~ .*[a-z].* ]] || [[ ! $pgpw_1 =~ .*[A-Z].* ]] || [[ ! $pgpw_1 =~ .*[0-9].* ]] || [[ ! ${#pgpw_1} -ge 10 ]] || [[ ! ${#pgpw_1} -le 64 ]]; then
        printf "Passwords must contain one lowercase letter, one uppercase letter, and one numerical digit, and must be between 10 and 64 characters long. Please ensure your password meets these requirements and try again.\n"
        return
    fi
    read -s -p "Re-enter chosen password: " pgpw_2
    printf "\n"
    if [[ $pgpw_1 != $pgpw_2 ]]; then
        printf "Password strings must match. Please try again.\n"
        return
    else
        pgpw=$pgpw_1
        return
    fi
}

script_setup() {
  agent_user="xfabric"
  agent_group="xfabric"
  prefix="/opt/integration/"
  pg_version="16"
  pg_hba_file="/var/lib/pgsql/$pg_version/data/pg_hba.conf"

  agent_version="@agentVersion@"
  build_number="@agentBuildNumber@"
  agent_id="@agentId@"
  agent_rpm_name="xfagent.x86_64.rpm"
  product_yum_server="repo.fortisoar.fortinet.com"
  tenant_id="@tenantid@"
  schema="@schema@"
  agent_config="# Copyright start
# Copyright (C) 2008 - 2023 Fortinet Inc.
# All rights reserved.
# FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
# Copyright end
rmq_public:
  host: @rmqPublicHost@
  sni: @rmqPublicSni@
  port: 5671
  vhost: @vhost@
  user: @user@
  ssl: true
  heartbeat: 120
  caCertPath: /opt/xf/configs/ca_cert.pem
  caChainBundle: /opt/xf/configs/ca_chain_bundle.pem
  clientCertFileName: /opt/xf/configs/client_cert.pem
  clientKeyFileName: /opt/xf/configs/client_key.pem
xf.instance.lwAgent: true
xf.instance.agentId: @agentId@
xf.instance.applianceUuid: @applianceUuid@
xf.instance.applianceKey: |
  @applianceKey@
database:
  db_external: false
  db_host: 127.0.0.1
  db_port: 5432
  db_ssl_cert_path: ''
  db_use_ssl: false
  db_user: cyberpgsql
  db_name: integration"
  ini_config="; Copyright start
; Copyright (C) 2008 - 2023 Fortinet Inc.
; All rights reserved.
; FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
; Copyright end

[application]
debug = false
app_host = @hostname@
THREAD_POOL_WORKER = 16
FSR_NULL = 'FSR_NULL'
conn_rpm_temp_dir = /opt/cyops-integrations/cyops_connector_rpm
auth_host = @hostname@
wf_host = @hostname@
integration_host = @hostname@

[logging]
app_log_file_path = /var/log/cyops/cyops-integrations/connectors.log
app_log_file_size = 1024*1024*100
app_log_file_backup_count = 10
root_logger_level = WARNING
connector_logger_level = INFO
django_logger_level = INFO
django_backend_logger_level = WARNING

#########
# Container config starts here
#########
[container]
# In K8s
# You can use the suffix m to mean milli.
# For example 100m CPU, 100 milliCPU, and 0.1 CPU are all the same.
# Precision finer than 1m is not allowed.
# For simplicity, do not use 'm' and instead specify in just number.
#
cpu = 0.2

#########
# Gunicorn config starts here
#########
[gunicorn]
# Can be debug, info, warning, error, critical
loglevel = debug
port = 9595
# In sec
graceful_timeout = 900
timeout = 600
max_requests = 2000
max_requests_jitter = 100"
  connectors_json='@connectorsJson@'
  ca_cert="@caCert@"
  ca_chain_bundle="@caChainBundle@"
  client_cert="@clientCert@"
  client_key="@clientKey@"
  pwd=$(pwd)
  f_script_name="xfagent-$product_version-$build_number.x86_64.rpm"

  pg_already_installed=0
  if rpm -q postgresql$pg_version-server; then
    $pg_already_installed=1
  fi
  
}

set_locale() {
  export LANG=en_US.UTF-8
  export LANGUAGE=en_US.UTF-8
  export LC_COLLATE=C
  export LC_CTYPE=en_US.UTF-8
}

install_agent_rpm() {
    # Download rpm file and install
    # Exit if the rpm file does not exist
    yum list xf-integration-agent-$agent_version-$build_number > /dev/null
    if [ $? -eq 0 ]
    then
        echo "Installing XFabric Agent package"
        yum -y --nogpgcheck install xf-integration-agent-$agent_version-$build_number
    else
        echo "Package xf-integration-agent-$agent_version-$build_number is not present in repository"
        exit 1
    fi
}

set_env_variables() {
    # Add misc environment variables
    echo export PYTHONUNBUFFERED=1 >> /etc/environment
    echo export PYTHONDONTWRITEBYTECODE=1 >> /etc/environment
    # export PYTHONUNBUFFERED=1
    # export PYTHONDONTWRITEBYTECODE=1

    # Add environment variables for agent detection
    echo export ENV="agent" >> /etc/environment
    echo export TENANT_ID="$tenant_id" >> /etc/environment
    echo export SCHEMA="$schema" >> /etc/environment
    # export ENV="agent"
    # export TENANT_ID="$tenant_id"
    # export SCHEMA="$schema"

}

add_extra_hosts() {
    local rmq_ip="69.167.115.175"
    local rmq_hostname="uat-agent-rmq.pool1.ca-west-1.xf.fortisoar.forticloud.com"
    printf "%s\t%s\n" "$rmq_ip" "$rmq_hostname" >> /etc/hosts
}

populate_config_files() {
    # Populate config files with the correct values
    echo "$agent_config" > /opt/integration/configs/agent/agent_config.yml
    echo "$ini_config" > /opt/integration/configs/agent/config.ini
    touch /opt/xf/configs/ca_cert.pem
    echo "$ca_cert" > /opt/xf/configs/ca_cert.pem
    touch /opt/xf/configs/client_cert.pem
    echo "$client_cert" > /opt/xf/configs/client_cert.pem
    touch /opt/xf/configs/ca_chain_bundle.pem
    echo "$ca_chain_bundle" > /opt/xf/configs/ca_chain_bundle.pem
    touch /opt/xf/configs/client_key.pem
    echo "$client_key" > /opt/xf/configs/client_key.pem
    sed -i.bak "s#%tenant_id%#$tenant_id#g" /opt/integration/package.d/gunicorn.service
    sed -i.bak "s#%schema%#$schema#g" /opt/integration/package.d/gunicorn.service
    rm -f /opt/integration/package.d/gunicorn.service.bak
}

populate_connectors_file() {
  touch /opt/integration/connectors/agent_connectors.json
  echo "$connectors_json" > /opt/integration/connectors/agent_connectors.json
}

disable_existing_postgres_repo() {
  # For Fix of following error
  # All matches were filtered out by modular filtering for argument: postgresql14-server
  sudo dnf -qy module disable postgresql
  yum clean all
}

add_enable_repo() {
  xf_app_repo="[xf-app]
name=XFabric Application Repository
baseurl=https://$product_yum_server/xf/1.0.0/agent/x86_64/
gpgcheck=0
enabled=1
sslverify=false"

  postgres_repo="[fsr-pgdg]
name=FortiSOAR PostgreSQL repository
baseurl=https://$product_yum_server/xf/1.0.0/third-party/postgres/$pg_version/rhel-\$releasever-\$basearch/
enabled=1
gpgcheck=1
gpgkey=https://$product_yum_server/xf/1.0.0/third-party/postgres/RPM-GPG-KEY-PGDG-$pg_version
sslverify=false"

  echo "$xf_app_repo" >/etc/yum.repos.d/xf-app.repo
  sudo chmod 644 /etc/yum.repos.d/xf-app.repo

  if [ "$pg_already_installed" -eq "0" ]; then
    echo "$postgres_repo" >/etc/yum.repos.d/fsr-pgdg.repo
    sudo chmod 644 /etc/yum.repos.d/fsr-pgdg.repo
  fi

  yum clean all
}

install_configure_postgres() {
  if [ "$pg_already_installed" -eq "0" ]; then
    echo "======================================================================================================"
    echo "Installing Postgres Server"
    echo "======================================================================================================"
    yum install -y postgresql$pg_version-server --nogpgcheck
    if [ $? -ne 0 ]; then
      echo "Unable to install PostgreSQL, Exiting the installer"
      echo "Validate connectivity with $product_yum_server or check host entry"
      exit 1
    fi
    #pg setup
    /usr/pgsql-$pg_version/bin/postgresql-$pg_version-setup initdb
  fi

  printf "database:\n  db_password: $(sudo -u $agent_user $prefix/.env/bin/python3 /opt/integration/manage.py manage_password encrypt "$pgpw")" > /opt/integration/configs/agent/db_config.yml

  systemctl enable postgresql-$pg_version

  #if [ $is_upgrade -ne 1 ]; then
  systemctl start postgresql-$pg_version.service

  create_pg_user_and_db
  setup_pg_hba

  systemctl restart postgresql-$pg_version.service
}

create_pg_user_and_db() {
  sudo -Hiu postgres psql -U postgres << XFABRICHEREDOCDELIM
SELECT pg_terminate_backend(pid) FROM pg_stat_activity WHERE datname = 'integration';

DROP DATABASE IF EXISTS integration;

DROP USER IF EXISTS cyberpgsql;
DO
\$XFabricPWDelim\$
BEGIN
    CREATE USER cyberpgsql WITH SUPERUSER PASSWORD '$pgpw';
END
\$XFabricPWDelim\$;

CREATE DATABASE integration OWNER cyberpgsql;
XFABRICHEREDOCDELIM
}

setup_pg_hba() {
  pg_hba="local   all             postgres                                md5

local   all             cyberpgsql                              md5

host    all             cyberpgsql             127.0.0.1/32     md5

host    all             cyberpgsql             ::1/128          md5"
    
  echo "$pg_hba" >/var/lib/pgsql/$pg_version/data/pg_hba.conf
  chown postgres:postgres /var/lib/pgsql/$pg_version/data/pg_hba.conf
  chmod 600 /var/lib/pgsql/$pg_version/data/pg_hba.conf
}

remove_repo() {
  rm -f /etc/yum.repos.d/xf-app.repo
  rm -f /etc/yum.repos.d/fsr-connectors.repo
  rm -f /etc/yum.repos.d/fsr-pgdg.repo
}

change_file_ownership() {
  # reset all file ownership to FortiSOAR user in case new files have been created
  chown -R $agent_user:$agent_group $prefix
  restorecon -R $prefix
  chown -R $agent_user:$agent_group /var/log/xfagent/
  restorecon -R $prefix /var/log/xfagent
  chown -R $agent_user:$agent_group /opt/xf/configs/
  restorecon -R $prefix /opt/xf/configs
}

misc_agent_setup() {
    mkdir -p /opt/xf/configs/
    touch /opt/xf/configs/.config_changed
    touch /var/log/xfagent/xfagent.log
    chmod 777 /var/log/xfagent/xfagent.log
    touch /var/log/gunicorn.log
    chown $agent_user:$agent_group /var/log/gunicorn.log
    chmod 766 /var/log/gunicorn.log
}

add_venv_to_path() {
  touch ~/.profile
  prof=$(<~/.profile)
  echo $prof
  if [ -d /opt/integration/.env/bin ] && [[ $prof != *"/opt/integration/.env/bin"* ]]; then
    echo "[ -d /opt/integration/.env/bin ] && PATH=\"/opt/integration/.env/bin:\$PATH\"" >> ~/.profile
  fi
}

database_migrate() {
  # run migration
  echo "STARTING DB MIGRATION"
  echo "ENV is $ENV"
  cd $prefix/
  sudo -u $agent_user $prefix/.env/bin/python3 /opt/integration/manage.py makemigrations
  sudo -u $agent_user $prefix/.env/bin/python3 /opt/integration/manage.py migrate
}

create_tenant() {
    sudo -u $agent_user /opt/integration/.env/bin/python /opt/integration/manage.py  create_xf_tenant --tenant_id $tenant_id  --schema_name $schema --name $tenant_id --active True
    sudo -u $agent_user /opt/integration/.env/bin/python /opt/integration/manage.py  create_self_agent_or_update_version --schema_name $schema
}

enable_agent_service() {
    systemctl enable /opt/integration/package.d/gunicorn.service
}

restart_service() {
  # restart service
  echo "restarting services"
  systemctl restart postgresql-$pg_version
  cd /opt/integration/
  source /etc/environment
  systemctl restart gunicorn.service
#   if [ $? -ne 0 ]; then
#     if [ $is_upgrade -eq 1 ]; then
#       sudo -u $agent_user /opt/cyops-integrations/.env/bin/python $prefix/agent_helper.py '{"code": 4, "status": "Agent service failed to start"}'
#       sudo -u $agent_user /opt/cyops-integrations/.env/bin/python $prefix/agent_helper.py '{"collect_log": true, "log_path": "/var/log/cyops"}'
#     fi
#   fi
}

install_connectors() {
  sudo -u $agent_user $prefix/.env/bin/python3 /opt/integration/manage.py install_agent_connectors --schema_name $schema --filepath /opt/integration/connectors/agent_connectors.json
}

update_build_info() {
  rpm_version="$(rpm -q --qf \"%{VERSION}\" xf-integration-agent)"
  rpm_build_number="$(rpm -q --qf \"%{RELEASE}\" xf-integration-agent)"
  echo "{\"release-version\":$rpm_version,\"build-number\":$rpm_build_number}" > /opt/build-info.json
}

install_gcc() {
  rpm -q gcc >/dev/null
  if [ $? -ne 0 ]; then
    echo "======================================================================================================"
    echo "attempting to install gcc"
    echo "======================================================================================================"
    yum install -y gcc >/dev/null 2>&1

    if [ 0 -ne $? ]; then
      echo "======================================================================================================"
      echo "Failed to install GCC rpm. Some of the FortiSOAR connectors may have a dependency on gcc."
      echo "It is recommended to install gcc by enabling the CentOS yum repositories on this instance and running 'yum install gcc -y'"
      echo "======================================================================================================"
    else
      echo "successfully installed gcc"
    fi
  fi
}

main() {
    local pgpw=""
    while [[ -z $pgpw ]]
    do
        get_password
    done
    mkdir -p /var/log/xfagent
    f_log="/var/log/xfagent/xfagent-$product_version-$time_stamp.log"
    touch $f_log
    f_file_descriptor_3="/dev/fd/3"
    exec 3>&1 1>>${f_log} 2>&1

    set -x
    {
      script_setup
      set_locale
      set_env_variables
      add_extra_hosts
      source /etc/environment
      add_enable_repo
      install_agent_rpm
      #add_venv_to_path
      update_build_info
      misc_agent_setup
      populate_config_files
      populate_connectors_file
      disable_existing_postgres_repo
      #add_product_yum_server
      install_configure_postgres
      remove_repo
      change_file_ownership
      database_migrate
      create_tenant
      database_migrate
      enable_agent_service
      restart_service
      install_gcc
      install_connectors
    } | awk '{ print strftime("[%Y-%m-%d %H:%M:%S]"), $0; fflush(); }' | tee -a $f_file_descriptor_3
    #exec > /dev/tty
    #printf "\n\n------------------\nXFabric Agent installed! /opt/integration/.env/bin has been added to PATH variable in ~/.profile.\nPlease run 'source ~/.profile' or log out and back in for this change to take effect\n------------------\n"
}

main
exit 0
