""" Copyright start
  Copyright (C) 2008 - 2023 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end """
import logging
from functools import reduce
from operator import and_, or_

from django.db.models import Q
from django_filters import CharFilter
from django_filters.rest_framework import FilterSet

from agents.models import Agent

logger = logging.getLogger(__name__)

def health_status_iexact(queryset, name, value):
    condition = Q(healthStatus__status__iexact=value)
    return queryset.filter(condition)

def health_status_in_list(queryset, name, value):
    values = value if isinstance(value, list) else value.split(',')
    condition = reduce(or_, [Q(healthStatus__status__iexact=value) for value in values])
    return queryset.filter(condition)


class AgentFilterSet(FilterSet):
    healthStatus = CharFilter(field_name='healthStatus', method=health_status_iexact)
    healthStatus__in = CharFilter(field_name='healthStatus', method=health_status_in_list)
    class Meta(object):
        model = Agent
        fields = {
        'name': ['exact', 'in', 'contains'],
        'active': ['exact']
        }
