from django.core.management.base import BaseCommand
from django.conf import settings
from django_tenants.utils import schema_context

from tenant.models import Tenant, Domain
from agents.models import Agent


class Command(BaseCommand):
    help = 'Creates an Agent object in the database, or updates its version if it already exists'

    def add_arguments(self, parser):
        schema_name = parser.add_argument('--schema_name')

    def handle(self, *args, **options):
        with schema_context(options.get('schema_name')):
            try:
                agent, created = Agent.objects.get_or_create(agentId=settings.SELF_ID, defaults={'name': 'Self Agent', 'isLocal': True},
                            applianceKey=settings.ALL_CONFIG.get('xf.instance.applianceKey'),
                            applianceUuid=settings.ALL_CONFIG.get('xf.instance.applianceUuid'),
                            version=settings.RELEASE_VERSION)
                if not created and agent.version != settings.RELEASE_VERSION:
                    agent.version = settings.RELEASE_VERSION
                    agent.save()
            except Exception as e:
                return self.stdout.write(self.style.ERROR('Error: Failed to create XF Agent: %s' % str(e)))
            finally:
                pass
            return self.stdout.write(self.style.SUCCESS('Successfully created XF Agent'))
