import uuid

from django.db import models
from django.db.models import JSONField


# Create your models here.
class TimeMixin(models.Model):
    """
    Provides some timestamps
    """
    createDate = models.DateTimeField(auto_now_add=True, blank=True, null=True, db_column='createdate')
    modifyDate = models.DateTimeField(auto_now=True, blank=True, null=True, db_column='modifydate')

    class Meta:
        abstract = True


class Agent(TimeMixin, models.Model):
    # replace agentId with uuid throughout service
    uuid = models.UUIDField(primary_key=True, default=uuid.uuid4, editable=False, max_length=150)
    agentId = models.CharField(max_length=1000, unique=True, db_column='agentid')  # Should have tenantID_{agentId}

    version = models.CharField(max_length=20, default='1.0.0')
    name = models.CharField(max_length=255, null=False)
    description = models.TextField(blank=True, null=True)

    agentType = models.CharField(max_length=50, blank=True, null=True, default='agent', db_column='agenttype')
    active = models.BooleanField(default=True)
    healthStatus = JSONField(default=dict, blank=True, null=True, db_column='healthstatus')
    syncTime = models.DateTimeField(blank=True, null=True, db_column='synctime')
    configStatus = models.CharField(max_length=1000, blank=True, null=True, db_column='configstatus')
    authType = models.CharField(max_length=50, blank=False, null=False, db_column='authtype')
    clientCertificate = models.TextField(blank=True, null=True, db_column='clientcertificate')
    clientKey = models.TextField(blank=True, null=True, db_column='clientkey')
    owners = models.ManyToManyField('identity.Team')
    isLocal = models.BooleanField(default=True, db_column='islocal')

    applianceKey = models.TextField(blank=True, null=True, db_column='appliancekey')
    applianceUuid = models.UUIDField(max_length=150, blank=True, null=True)
