""" Copyright start
  Copyright (C) 2008 - 2023 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end """

from packaging.version import Version
from django.conf import settings
from rest_framework import serializers

from agents.models import Agent
from query.serializers import DynamicFieldsModelSerializer


class AgentSerializer(serializers.ModelSerializer):
    upgrade_available = serializers.SerializerMethodField('is_upgrade_available')
    class Meta:
        model = Agent
        fields = '__all__'
    
    def is_upgrade_available(self, obj):
        xf_version = Version(settings.RELEASE_VERSION)
        agent_version = Version(obj.version)
        if xf_version > agent_version:
            return True
        return False


class AgentReadOnlySerializer(serializers.ModelSerializer):
    class Meta:
        model = Agent
        exclude = ('applianceKey', 'clientKey', 'clientCertificate')


class AgentQuerySerializer(DynamicFieldsModelSerializer):
    upgrade_available = serializers.SerializerMethodField('is_upgrade_available')
    class Meta:
        model = Agent
        fields = '__all__'
    def is_upgrade_available(self, obj):
        xf_version = Version(settings.RELEASE_VERSION)
        agent_version = Version(obj.version)
        if xf_version > agent_version:
            return True
        return False
