import logging
from django.conf import settings
from django.db import connection
from xfutils.utils import generate_interservice_jwt_token, api_call
from agents.exceptions import *
from agents.models import Agent
from constants.error_message import xf_integration_50, xf_integration_51, xf_integration_52

logger = logging.getLogger(__name__)

def has_room_for_agent():
    current_agent_count = Agent.objects.count()

    try:
        url = f"https://{settings.AUTH_HOST}/api/auth/licenses/"
        token = generate_interservice_jwt_token(settings.ALL_CONFIG, tenantid=connection.get_tenant().tenant_id)
        headers = {"Authorization": f"Bearer {token}"}
        response = api_call(url, method='GET', headers=headers, verify=False)
    except Exception as e:
        raise LicenseFetchFailed(detail=xf_integration_50.format(e))
    licenses = response.get('hydra:member')
    if len(licenses) > 1:
        raise TooManyLicenses(xf_integration_51.format(connection.get_tenant().tenant_id))
    max_agents = licenses[0].get('info', {}).get('number_of_agents', None)
    if max_agents and current_agent_count > max_agents:
        raise AgentLimitReached(xf_integration_52)
    return True