import logging
from django.db import IntegrityError

from connectors.core.constants import REMOTE_STATUS
from connectors.models import Connector
from connectors.serializers import ConnectorDetailSerializer

logger = logging.getLogger('connectors')


def create_connector_object(connector, agent, status='in-progress',
                            remote_status=REMOTE_STATUS.get('installation_in_progress')):
    conn_data = {
        'name': connector.get('name'),
        'version': connector.get('version'),
        'agent': agent.agentId,
        'label': connector.get('label'),
        'status': status,
        'remote_status': {'status': remote_status}
    }

    conn_instance = Connector.objects.filter(name=connector.get('name'), version=connector.get('version'),
                                             agent_id=agent.agentId).first()
    if conn_instance and conn_instance.remote_status.get('status') in ['in-progress',
                                                                       'installation_in_progress-in-progress']:
        raise IntegrityError('Connector you are trying to install is already in progress')
    elif conn_instance and conn_instance.remote_status.get('status') == 'finished':
        message = 'Connector you are trying to install is already installed Connector :: {0} v{1}'.format(
            connector.get('name'), connector.get('version'))
        raise IntegrityError({'message': message})

    connector_serializer = ConnectorDetailSerializer(conn_instance, data=conn_data)
    connector_serializer.is_valid(raise_exception=True)
    connector_serializer.save()
    conn_data = connector_serializer.data

    ##
    ## TODO: Implement custom connector implementation
    ##
    return conn_data


def is_connector_installable_on_agent(name, version, agent_id):
    connector_qs = Connector.objects.filter(name=name, version=version, agent_id=agent_id)
    if connector_qs.exists():
        # If connector is in progress and install request comes again
        conn_instance = connector_qs.first()
        if conn_instance.remote_status.get('status') in ['in-progress', 'installation_in_progress-in-progress']:
            msg = f'Installation of Connector {name} version {version} is already in progress'
            logger.info(msg)
            return False, msg
        elif conn_instance.status == 'installed' and conn_instance.remote_status.get('status', None) == REMOTE_STATUS.get('finished'):
            msg = f'Connector {name} version {version} is already installed'
            logger.info(msg)
            return False, msg
    return True, None