""" Copyright start
  Copyright (C) 2008 - 2023 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end """
import json
import logging
import time

from django.conf import settings
from pika.exceptions import ChannelClosed

# from utils.rabbitmq import RabbitMQPublisher
from connectors.core.constants import *

logger = logging.getLogger("connectors")


def publish_audit_log(data, exchange_name="texchange.cyops.sdk", routing_key="key.sdk.audit",
                      retry_count=settings.PUBLISH_MESSAGE_RETRY_COUNT):
    try:
        rmq = RabbitMQPublisher()
        rmq.publish(exchange_name,
                    routing_key,
                    body=json.dumps(data),
                    content_type="application/json",
                    retry_count=retry_count,
                    delivery_mode=1
                    )
    except ChannelClosed as e:
        logger.error('Audit Service Message Channel is not yet Created Error:: %s' % str(e))


def publish_audit_and_notify(data, method, module, exchange, routing_key, title="", request=None, rbac_info=None,
                             entity_uuid=None):
    try:
        if settings.LW_AGENT:
            return
        operation = 'Create' if method == 'POST' else 'Update' if method == 'PUT' else 'Delete' if method == 'DELETE' else method
        user_name = ""
        user_uuid = ""
        record_uuid = ""
        if request:
            user_uuid = request.data.get('rbac_info', {}).get('user_iri')
            user_uuid = user_uuid.split('/')[-1] if user_uuid else None
            source = request.data.get('rbac_info', {}).get('source', settings.SELF_ID)
            record_iri = request.data.get('audit_info', {}).get('record_iri')
            module = record_iri.split('/')[-2] if record_iri else module
            record_uuid = record_iri.split('/')[-1] if record_iri else ""
        elif rbac_info:
            user_iri = rbac_info.get('user_iri')
            source = rbac_info.get('source', settings.SELF_ID)
            user_uuid = user_iri.split('/')[-1] if user_iri else None
        else:
            user_name = "Master"
            source = settings.SELF_ID
            if settings.MASTER_ID:
                user_name = "Agent"
        if user_uuid is None:
            return
        if entity_uuid:
            record_uuid = entity_uuid

        audit_data = {
            "data": data,  # Format for data {'data':data, 'message': message, 'status': status}
            "operation": operation,
            "source": source,
            "component": "agent",
            "entityType": module.lower(),
            "entitySingularName": "" if record_uuid and not entity_uuid else module.rstrip('s'),
            "entityUuid": record_uuid,
            "userId": user_uuid,
            "user": user_name,
            'transactionDate': int(round(time.time() * 1000)),
            'title': title
        }
        publish_audit_log([audit_data], exchange_name=exchange, routing_key=routing_key, retry_count=1)

    except Exception as e:
        logger.error("Error while auditing connector action execution %s", str(e))


def audit_connector_action(audit_data, status):
    try:
        if settings.LW_AGENT:
            return
        user_iri = audit_data.get("user_iri")
        user_id = user_iri.split('/')[-1] if user_iri else ""

        record_iri = audit_data.get("record_iri")
        record_type = record_iri.split('/')[-2] if record_iri else ""
        record_uuid = record_iri.split('/')[-1] if record_iri else ""
        audit_data['data']['status'] = status

        audit_data = {
            "operation": CONNECTOR_AUDIT_OPERATIONS.get("execute"),
            "component": "sdk",
            "entityType": record_type,
            "entityUuid": record_uuid,
            "userId": user_id,
            "source": audit_data.get("source"),
            "transactionDate": int(round(time.time() * 1000)),
            "data": audit_data['data']
        }
        publish_audit_log([audit_data])
    except Exception as e:
        logger.error("Error while auditing connector action execution %s", str(e))


def audit_connector_functions(data, operation, status, module='Connector', audit_message='', rbac_info={}):
    try:
        if not settings.LW_AGENT:
            operation = operation.lower()
            entity_uuid = None
            audit_data = {
                'data': data,
                'status': status,
                'has_start_event': False,
            }
            # For identifying the events to be updated on the previous events
            # Eg: Install started then second event install completed
            if not rbac_info and not rbac_info.get('user_iri'):
                audit_data.update({'has_start_event': True})
            connector_name = data.get('name') or data.get('connector')
            connector_version = data.get('version')
            agent_id = data.get('agent') or data.get('agent_id') or settings.SELF_ID
            if connector_name and connector_version:
                entity_uuid = '{0}_{1}_{2}'.format(connector_name, connector_version, agent_id)
            audit_method = CONNECTOR_AUDIT.get(module.lower(), CONNECTOR_AUDIT_OPERATIONS).get(operation, '')
            publish_audit_and_notify(audit_data, audit_method, module, 'texchange.cyops.sdk', 'key.sdk.audit',
                                     audit_message, rbac_info=rbac_info, entity_uuid=entity_uuid)
    except Exception as e:
        logger.error("Error while auditing connector action execution %s", str(e))
