from django.conf import settings
from agents.constants import REMOTE_AGENT_OPERATIONS
from connectors.core.connector import logger, ConnectorError
from connectors.core.constants import REMOTE_CONNECTOR_OPERATIONS, REMOTE_STATUS
from connectors.utils import send_connector_operations_ack


class ConnectorOperation:
    action_mapping = {
        REMOTE_CONNECTOR_OPERATIONS.get('connector_install'): 'install_connector',
        REMOTE_CONNECTOR_OPERATIONS.get('connector_uninstall'): 'uninstall_connector',
        REMOTE_CONNECTOR_OPERATIONS.get('connector_update'): 'connector_detail_update',
        REMOTE_CONNECTOR_OPERATIONS.get('config_create'): 'create_connector_config',
        REMOTE_CONNECTOR_OPERATIONS.get('config_delete'): 'delete_connector_config',
        REMOTE_CONNECTOR_OPERATIONS.get('config_update'): 'update_connector_config',
        REMOTE_CONNECTOR_OPERATIONS.get('connector_execution'): 'connector_execution',
        REMOTE_CONNECTOR_OPERATIONS.get('connector_healthcheck'): 'connector_healthcheck',
        REMOTE_AGENT_OPERATIONS.get('agent_healthcheck'): 'agent_healthcheck'
    }

    def handle_response(self, data, status, operation, message="", tenant=None):
        if settings.LW_AGENT:
            data.update({
                'status': status,
                'message': message
            })
            send_connector_operations_ack(data, operation, tenant)
        elif status == REMOTE_STATUS.get('failed'):
            raise ConnectorError(message)

    def install_connector(self, data, request=None):
        raise ConnectorError('Operation {0} is not implemented for connector'.format(
            REMOTE_CONNECTOR_OPERATIONS.get('connector_install')))

    def uninstall_connector(self, data, request=None):
        raise ConnectorError('Operation {0} is not implemented for connector'.format(
            REMOTE_CONNECTOR_OPERATIONS.get('connector_uninstall')))

    def connector_detail_update(self, data, request=None):
        raise ConnectorError('Operation {0} is not implemented for connector'.format(
            REMOTE_CONNECTOR_OPERATIONS.get('connector_update')))

    def create_connector_config(self, data, request=None):
        raise ConnectorError('Operation {0} is not implemented for connector'.format(
            REMOTE_CONNECTOR_OPERATIONS.get('config_create')))

    def connector_execution(self, data, request=None):
        raise ConnectorError('Operation {0} is not implemented for connector'.format(
            REMOTE_CONNECTOR_OPERATIONS.get('connector_execution')))
    
    def connector_healthcheck(self, data, request=None):
        raise ConnectorError('Operation {0} is not implemented for connector'.format(
            REMOTE_CONNECTOR_OPERATIONS.get('connector_healthcheck')))
    
    def update_connector_config(self, data, request=None):
        raise ConnectorError('Operation {0} is not implemented for connector'.format(
            REMOTE_CONNECTOR_OPERATIONS.get('config_update')))
    
    def delete_connector_config(self, data, request=None):
        raise ConnectorError('Operation {0} is not implemented for connector'.format(
            REMOTE_CONNECTOR_OPERATIONS.get('config_delete')))