""" Copyright start
  Copyright (C) 2008 - 2023 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end """
import json
import os

from django.http import Http404
from rest_framework import status
from rest_framework.response import Response

from agents.models import Agent
from audit.audit import publish_audit_and_notify
from connectors.core.connector import logger
from connectors.core.constants import *
from connectors.models import Connector, Configuration
from connectors.serializers import ConnectorConfigurationSerializer, ConnectorDetailSerializer
from connectors.utils import broadcast_message
from connectors.utils import (omit_password,
                              get_configuration, on_active_status_update,
                              get_connector_path)
from constants.error_message import *
from postman.utils.constants import *

def get_configuration_obj(pk):
    try:
        if pk.isdigit():
            obj = Configuration.objects.filter(pk=pk).first()
        else:
            obj = Configuration.objects.filter(config_id=pk).first()

        # hide passwords in connector configs
        if obj:
            serialized_obj = ConnectorConfigurationSerializer(obj).data
            serialized_obj.pop('connector', None)
            serialized_obj.pop('agent', None)
            serialized_obj.pop('teams', None)

            omit_dict = {
                'configuration': serialized_obj,
                'config_schema': obj.connector.config_schema
            }
            omit_password(omit_dict)
            obfuscated_obj = Configuration(**omit_dict['configuration'])
            obfuscated_obj.connector = obj.connector
            obfuscated_obj.agent = obj.agent
            obfuscated_obj.teams.set(obj.teams.all())

            return obfuscated_obj
        return obj
    except Configuration.DoesNotExist:
        raise Http404('No matching config by id %s exists.' % pk)


def get_agent_obj(agent_id):
    return Agent.objects.filter(agentId=agent_id).first()


def brodacast_connector_operation_message(conn_id, command, status, name=None, version=None, agent_id=None,
                                          message=None):
    conn_object = Connector.objects.filter(id=conn_id).first()
    if conn_object:
        name = conn_object.name
        version = conn_object.version
        agent_id = conn_object.agent.agentId
    else:
        return
    broadcast_data = {
        'name': name,
        'version': version,
        'status': {
            'status': REMOTE_STATUS.get('finished') if status == 'Success' else REMOTE_STATUS.get('failed'),
            'message': message if message else conn_object.install_result.get('message')
        },
        'command': command,
        'agent': agent_id,
        'rpm_installed': conn_object.metadata.get('rpm_installed', False)
    }

    ack_action = install_connector_ack if command == 'install' else uninstall_connector_ack

    if command == 'install' and status == 'Success':
        connector_path = get_connector_path(conn_object.name, conn_object.version)
        info_path = os.path.join(connector_path, 'info.json')
        with open(info_path) as jFile:
            info_content = json.load(jFile)
        if Configuration.objects.filter(connector=conn_object.id).exists():
            config_instance = Configuration.objects.filter(connector=conn_object.id)
            config_serializer = ConnectorConfigurationSerializer(config_instance, many=True)
            broadcast_data.update({'config': config_serializer.data})
        broadcast_data.update({'info': info_content})

    broadcast_message(broadcast_data, ack_action, 'receiveRemoteConnectorInstructionRequest', 'POST',
                      conn_object.agent)
