""" Copyright start
  Copyright (C) 2008 - 2023 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end """
import os

from django.conf import settings
from django.core.management.base import BaseCommand
from django_tenants.utils import tenant_context

from connectors.views import import_connector
from tenant.utils import _get_tenant


class Command(BaseCommand):
    help = 'Installing connectors'

    def add_arguments(self, parser):
        parser.add_argument(
            '-delete_files',
            '--delete_files',
            action='store_true',
            dest='delete_files',
            help='Re-import connector info',
        )
        parser.add_argument(
            '-cs-propagated',
            '--cs-propagated',
            action='store_true',
            dest='cs_propagated',
            help='Re-import connector info',
        )
        parser.add_argument('--tenant_id', type=str, help='name of the connector', )

    def handle(self, *args, **options):
        tenant_id = options.get('tenant_id')
        with tenant_context(_get_tenant(tenant_id)):
            try:
                delete_files = options.get('delete_files', False)
                cs_propagated = options.get('cs_propagated', False)
                for root, dirname, files in os.walk(settings.CONNECTORS_DIR):
                    for file in files:
                        filename, file_extension = os.path.splitext(file)
                        if file_extension == '.tgz':
                            isReserved = False
                            if filename in settings.CONNECTORS_RESERVED:
                                isReserved = True
                            try:
                                result = import_connector(os.path.join(settings.CONNECTORS_DIR, '%s' % file), True,
                                                          isReserved, delete_uploaded_file=delete_files,
                                                          cs_propagated=cs_propagated)
                            except Exception as exp:
                                self.stdout.write(
                                    self.style.ERROR('Importing of connector %s failed: %s' % (file, str(exp))))
                                continue
            except Exception as exp:
                self.stdout.write(self.style.ERROR('Error importing all the connectors: %s' % str(exp)))
