""" Copyright start
  Copyright (C) 2008 - 2023 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end """
from django.core.management.base import BaseCommand

from connectors.views import ConnectorDetail


class Command(BaseCommand):
    help = 'Delete connector'

    def add_arguments(self, parser):
        parser.add_argument('name', type=str)
        parser.add_argument('version', type=str)
        parser.add_argument(
            '-cs-propagated',
            '--cs-propagated',
            action='store_true',
            dest='cs_propagated',
            help='Re-import connector info',
        )

    def handle(self, *args, **options):
        name = options.get('name')
        version = options.get('version')
        cs_propagated = options.get('cs_propagated')
        try:
            detail_view = ConnectorDetail()
            detail_view.kwargs = {'name': name, 'version': version, 'system_delete': True,
                                  'cs_propagated': cs_propagated}
            response = detail_view.destroy(options)
            if not response:
                self.stdout.write(self.style.SUCCESS('Successfully deleted connector.'))
            if response.status_code in [200, 204]:
                self.stdout.write(self.style.SUCCESS('Successfully deleted connector.'))
            else:
                self.stdout.write(self.style.ERROR('%s: %s' % (response.status_code, str(response.data))))
        except Exception as exp:
            self.stdout.write(self.style.ERROR('Failed to delete the connector: %s' % str(exp)))
