""" Copyright start
  Copyright (C) 2008 - 2023 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end """
import json
import shutil
from django_tenants.utils import tenant_context

from django.conf import settings
from django.core.management.base import BaseCommand

# from postman.models import Agent
from connectors.core.constants import REMOTE_CONNECTOR_OPERATIONS, REMOTE_STATUS
from connectors.models import Connector
from connectors.connector_self_operations import SelfOperations
from connectors.utils import (untar_connector_tgz, get_validated_info_file, import_connector, install_connector_files, send_connector_operations_ack)
from tenant.models import Tenant

CONNECTOR_PATH = settings.DJANGO_ROOT


class Command(BaseCommand):
    help = 'Installing connector on agent'
    suppressed_base_arguments = {'--version', '-v', '--settings', '--pythonpath', '--traceback', '--no-color', '--force-color'}
    def add_arguments(self, parser):
        parser.add_argument('path', type=str, help='Path of the connector .tgz file', )
        parser.add_argument('name', type=str, help='Name of connector')
        parser.add_argument('version', type=str, help='Version of the connector')

    def handle(self, *args, **options):
        # call the install_connector() from self ops
        #try:
        if not settings.LW_AGENT:
            print("This command is only available on XF Agents for installing custom connectors")
            return
        
        tenant = Tenant.objects.get(tenant_id=settings.TENANT_ID)
        with tenant_context(tenant):
            data_dict = {
                'name': options.get('name'),
                'version': options.get('version'),
                'agent': settings.SELF_ID,
                'replace': True,
                'file': options.get('path'),
                'request_data': {
                    'tenantid': settings.TENANT_ID,
                    'username': settings.SELF_ID,
                    'useruuid': settings.ALL_CONFIG.get('xf.instance.applianceUuid'),
                    #TODO: proper logic for subtenant mapping
                    'subtenantid': settings.TENANT_ID,
                    #TODO: proper logic for teams mapping?
                    'teams': [],
                    #if the audit message has no origin element audit servuce throws error
                    'META': {"REMOTE_ADDR": f"agent_{settings.SELF_ID}"}
                },
                'trust_api': False, # remove here and in install_connector()
                'rbac_info': None # remove here and in install_connector()
            }
            ops_obj = SelfOperations()
            try:
                result = ops_obj.install_connector(data_dict, custom_connector=True)
            except Exception as e:
                print(f"Error installing connector from file {options.get('path')}:\n{str(e)}\nSee /var/log/xfagent/xfagent.log for more information.")
        return
        #except Exception as e:
        #    print(f"Encountered the following error when installing connector:\n{e}")

    
    def _check_if_already_installing(self, name, version):
        connector_qs = Connector.objects.filter(name=name, version=version)
        if connector_qs.exists():
            # If connector is in progress and install request comes again
            conn_instance = connector_qs.first()
            if conn_instance.remote_status.get('status') in ['in-progress', 'installation_in_progress-in-progress']:
                return True
        return False