""" Copyright start
  Copyright (C) 2008 - 2023 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end """
import os
import re
import shutil

from django.conf import settings
from django.core.management import BaseCommand

from connectors.models import Connector
from connectors.serializers import ConnectorListSerializer
from connectors.views import import_connector


class Command(BaseCommand):
    help = (
        "Helps to update all connector in the database from its latest info.json."
    )

    def is_connector_installed(self, connector_name, connector_version, connectors):
        for connector in connectors:
            if connector_name == connector.get('name') and connector_version.replace('_', '.') == connector.get(
                    'version'):
                return True

    def handle(self, **options):
        try:
            connector_folder_mapping = {}
            connector_folder_regex = '.*_[0-9]_[0-9]_[0-9]'
            for folder_name in os.listdir(settings.CONNECTOR_INSTALLATION_DIR):
                if re.match(connector_folder_regex, folder_name):
                    conn_name = re.sub('_[0-9]_[0-9]_[0-9]', '', folder_name)
                    conn_version = re.findall('[0-9]_[0-9]_[0-9]', folder_name)[0]
                    if conn_name in connector_folder_mapping:
                        if int(conn_version.replace('_', '')) > int(
                                connector_folder_mapping[conn_name].replace('_', '')):
                            connector_folder_mapping[conn_name] = conn_version
                    else:
                        connector_folder_mapping[conn_name] = conn_version
            all_installed_connectors_query = Connector.objects.filter(agent=settings.SELF_ID)
            all_installed_connectors = ConnectorListSerializer(all_installed_connectors_query, many=True).data
            for name, version in connector_folder_mapping.items():
                if self.is_connector_installed(name, version, all_installed_connectors):
                    continue
                if name not in settings.CONNECTORS_RESERVED:
                    folder_path = os.path.join(settings.CONNECTOR_INSTALLATION_DIR, name + '_' + version)
                    try:
                        shutil.rmtree(folder_path)
                    except Exception as e:
                        pass
                else:
                    connector_base_path = '/opt/cyops-connector-' + name
                    tgz_file_path = os.path.join(connector_base_path, name + '.tgz')
                    import_connector(tgz_file_path, replace=True, isReserved=True, rpm_installed=True)

            self.stdout.write(self.style.SUCCESS('Successfully re-imported all connectors'))

        except Exception as e:
            self.stdout.write(
                self.style.ERROR('Error occurred while reimporting the connectors ERROR :: {0}'.format(str(e))))
