""" Copyright start
  Copyright (C) 2008 - 2023 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end """
import json
import os
import requests

from cshmac.requests import HmacAuth
from django.conf import settings

from connectors.core.connector import logger


def configure_proxy(system_settings=None):
    try:
        config = {}
        if not system_settings:
            url = '/api/3/proxy'
            url = settings.CRUD_HUB_URL + url
            body = None
            auth = HmacAuth(url, 'GET', settings.APPLIANCE_PUBLIC_KEY,
                            settings.APPLIANCE_PRIVATE_KEY,
                            json.dumps(body))
            try:
                response = requests.request('GET', url, auth=auth, json=body, verify=False)
                from xfutils.utils import maybe_json_or_raise
                system_settings = maybe_json_or_raise(response)
            except Exception as e:
                logger.error('Error while fetching proxy details %s', str(e))
                system_settings = None

        if system_settings:
            for k, v in system_settings.get('proxies', {}).items():
                config.update({k: v.get('value', '')})

            NO_PROXY = ','.join(system_settings.get('noProxy', []))
            if NO_PROXY:
                logger.info('Setting env variable %s', 'NO_PROXY')
                os.environ['NO_PROXY'] = NO_PROXY

            for el in system_settings.get('OTHER', []):
                key = list(el.keys())[0]
                logger.info('Setting env variable %s', key)
                os.environ[key] = el.get(key)

            for k, v in config.items():
                if v and system_settings.get('proxies', {}).get(k, {}).get('is_enabled'):
                    logger.info('Setting env variable %s', k)
                    os.environ[k] = v
                else:
                    logger.info('Clearing env variable %s', k)
                    os.environ.pop(k, None)
    except Exception as e:
        logger.exception('Error while updating proxy settings %s', str(e))
