import uuid

from django.db import models
from django.db.models import JSONField

# Create your models here.
class ContentHub(models.Model):
    TYPE_CHOICES = [
        ('SolutionPack', 'Solution Pack'),
        ('Connector', 'Connector'),
        ('Widget', 'Widget')
    ]
    STATUS_CHOICES = [
        ('Installed', 'Installed'),
        ('Not Installed', 'Not Installed'),
    ]
    uuid = models.UUIDField(primary_key=True, default=uuid.uuid4, editable=True, max_length=150)
    name = models.CharField(max_length=200, unique=True)
    description = models.CharField(max_length=1000, blank=True, null=True)
    version = models.CharField(max_length=20)
    type = models.CharField(max_length=200)
    installed = models.BooleanField(default=False)
    label = models.CharField(max_length=200, blank=True, null=True)
    publishedDate = models.DateTimeField(auto_now_add=True, blank=True, null=True)
    lastUpdated = models.DateTimeField(auto_now_add=True, blank=True, null=True)
    availableVersions = JSONField(default=list, blank=True, null=True)
    category = JSONField(default=list, blank=True, null=True)
    dependencies = JSONField(default=list, blank=True, null=True)
    help = models.CharField(max_length=500, blank=True, null=True)
    tags = JSONField(default=list, blank=True, null=True)
    prerequisite = JSONField(default=list, blank=True, null=True)
    fsrMinCompatibility = models.CharField(max_length=20, blank=True, null=True)
    featuredTags = JSONField(default=list, blank=True, null=True)
    featured = models.BooleanField(blank=True, null=True, default=True)
    contents = JSONField(default=dict, blank=True, null=True)
    dependentSolutionPacks = JSONField(default=list, blank=True, null=True)
    releaseNotes = models.CharField(max_length=1000, blank=True, null=True)
    status = models.CharField(max_length=100, choices=STATUS_CHOICES, default='Not Installed')
    iconLarge = models.TextField(blank=True, null=True)
    publisher = models.CharField(max_length=200, blank=True, null=True)
    certified = models.BooleanField(blank=True, null=True)
    buildNumber = models.IntegerField(blank=True, null=True)
    infoPath = models.CharField(max_length=500, blank=True, null=True)
    supportInfo = models.TextField(blank=True, null=True)
    recordTags = JSONField(default=list, blank=True, null=True)
    scm = JSONField(default=dict, blank=True, null=True)
    subTitle = models.CharField(max_length=500, blank=True)
    metadata = JSONField(default=dict, blank=True, null=True)
    operations = JSONField(default=dict, blank=True, null=True)
    certifiedBy = models.CharField(max_length=200, blank=True, default="")
    author = models.CharField(max_length=200, blank=True, default="")
    supported = models.BooleanField(blank=True, default=False)

    # class Meta:
    #     unique_together = (('name', 'version'),)

    def __str__(self):
        return self.name