import logging
import uuid
from threading import Thread

import jmespath

from django_filters.rest_framework import DjangoFilterBackend
from rest_framework import viewsets, status, filters
from rest_framework.response import Response
from rest_framework.decorators import action
from django.db.models import Q
from django.conf import settings

from contenthub.serializers import ContentHubSerializer
from contenthub.models import ContentHub
from contenthub.utils import sync_connectors
from xfutils.query.filtersets import get_query

logger = logging.getLogger(__name__)

class ContentHubView(viewsets.ModelViewSet):
    """
    Viewset for Content Hub
    """
    queryset = ContentHub.objects.all()
    serializer_class = ContentHubSerializer
    filter_backends = (filters.OrderingFilter,
                       filters.SearchFilter,
                       DjangoFilterBackend,)
    ordering_fields = ('name', 'description')
    search_fields = ('$name', '$description', '$type')
    filter_fields = ('name', 'description', 'type')

    def get_queryset(self):
        installed = self.request.GET.get('installed')
        search_value = self.request.data.get('search')
        # Todo remove if UI sends the search key in url
        if search_value:
            self.request.query_params._mutable = True
            self.request.query_params.appendlist('search', search_value)
            self.request.query_params._mutable = False
        queryset = ContentHub.objects.filter()
        if installed and installed.lower() == 'true':
            pass
        return self.filter_queryset(queryset)

    def content_hub_list(self, request, *args, **kwargs):
        queryset = self.filter_queryset(self.get_queryset())
        filter_data = request.data
        queryset = get_query(filter_data, queryset)
        page = self.paginate_queryset(queryset)
        if page is not None:
            if filter_data.get('aggregates'):
                return Response({'data': page})
            serializer = self.get_serializer(page, many=True)
            return self.get_paginated_response(serializer.data)

        serializer = self.get_serializer(queryset, many=True)
        return Response(serializer.data)

    @action(methods=['post'], detail=False)
    def sync(self, *args, **kwargs):
        sync_connectors()
        return Response({"Sync successfully"})

    def content_hub_detail(self, request, *args, **kwargs):
        name = kwargs.get('name')
        version = kwargs.get('version')
        if version == 'latest':
            queryset = ContentHub.objects.filter(name=name).order_by('-version').first()
        else:
            queryset = ContentHub.objects.filter(name=name, version=version).first()
        serializer = ContentHubSerializer(queryset)
        return Response(serializer.data)
