""" Copyright start
  Copyright (C) 2008 - 2023 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end """
import argparse
import fnmatch
import os
from subprocess import check_call

from Cython.Build import cythonize

excluded_files = ['__init__.py', 'wsgi.py', 'cythonize.py', 'settings.py', 'manage.py',
                  'consumer.py', 'setupsdk.py', 'cs_sdk.py', 'crudhub.py', 'agent_helper.py']


def is_excluded(filename):
    return os.path.basename(filename) in excluded_files


app_dir = None

ap = argparse.ArgumentParser()
ap.add_argument("-a", "--app", required=False, help="app to cythonize. all apps if not specified")
args, unknown = ap.parse_known_args()
args = vars(args)

if args['app']:
    app_dir = args['app']

current_dir = os.path.dirname(os.path.abspath(__file__))
if app_dir:
    current_dir = os.path.join(current_dir, app_dir)
matches = []
for root, dirnames, filenames in os.walk(current_dir):
    if not ('.env' in root or 'migrations' in root or 'template' in root):
        for filename in fnmatch.filter(filenames, '*.py'):
            matches.append(os.path.join(root, filename))

files = [fn for fn in matches if not is_excluded(fn)]

for filepath in files:
    file_no_ext = filepath.rsplit('.', 1)[0]
    cythonize(filepath, language_level="3")
    check_call(['gcc', '-shared', '-pthread', '-fPIC', '-fwrapv', '-O2',
                '-Wall', '-fno-strict-aliasing',
                '-I/usr/include/python3.9', '-o',
                '{}.so'.format(file_no_ext), '{0}.c'.format(file_no_ext)])
    os.remove(filepath)
    os.remove(file_no_ext + '.c')
