""" Copyright start
  Copyright (C) 2008 - 2023 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end """
from django_filters.rest_framework import DjangoFilterBackend
from rest_framework import filters
from rest_framework import status
from rest_framework import viewsets
from rest_framework.response import Response

from connectors.core.connector import logger
from data_import.filtersets import DataImportFilterSet
from data_import.models import DataImport
from data_import.serializer import DataImportSerializer


class DataImportView(viewsets.ModelViewSet):
    """
    A simple ViewSet for Data Import.
    """
    queryset = DataImport.objects.all()
    serializer_class = DataImportSerializer

    filter_backends = (filters.OrderingFilter,
                       filters.SearchFilter,
                       DjangoFilterBackend,)

    filter_class = DataImportFilterSet

    ordering_fields = ('id', 'name',)
    search_fields = ('$name', '$id', '$connector',)

    def get_queryset(self):
        queryset = DataImport.objects.all()
        name = self.request.GET.get('connector_name')
        version = self.request.GET.get('connector_version')
        if name and version:
            queryset = DataImport.objects.filter(connector__name=name, connector__version=version)
        return queryset

    def create(self, request, *args, **kwargs):
        data = request.data
        config_id = data.get('configuration')
        try:
            if DataImport.objects.filter(configuration=config_id).exists():
                instance = DataImport.objects.get(configuration=config_id)
                serializer = self.get_serializer(instance, data=request.data, partial=True)
                serializer.is_valid(raise_exception=True)
                self.perform_update(serializer)
            else:
                if not request.data.get('owners'):
                    request.data['owners'] = ['public']
                request.data['previous_config_id'] = None
                serializer = self.get_serializer(data=request.data)
                serializer.is_valid(raise_exception=True)
                self.perform_create(serializer)
            headers = self.get_success_headers(serializer.data)
            return Response(serializer.data, status=status.HTTP_201_CREATED, headers=headers)
        except Exception as e:
            error_message = 'Error occured while upserting data import ERROR :: {0}'.format(str(e))
            logger.error(error_message)
            return Response({"message": error_message}, status=status.HTTP_400_BAD_REQUEST)
