import uuid as uuid

from django.contrib.auth.base_user import AbstractBaseUser
from django.db import models

from identity.manager import UserManager
from tenant.models import Tenant


class User(AbstractBaseUser):
    uuid = models.UUIDField(primary_key=True, default=uuid.uuid4, editable=False)
    username = models.EmailField()  # email
    first_name = models.CharField(max_length=100)
    last_name = models.CharField(max_length=100)
    tenant = models.ForeignKey(Tenant, on_delete=models.CASCADE, null=True, blank=True)
    objects = UserManager()

    class Meta:
        unique_together = [['username', 'tenant']]

    def __str__(self):
        return f'{self.username}'


class Team(models.Model):
    uuid = models.UUIDField(primary_key=True, default=uuid.uuid4, editable=False)
    name = models.CharField(max_length=100, null=True, blank=True)
    members = models.ManyToManyField(User)
    tenant = models.ForeignKey(Tenant, on_delete=models.CASCADE, null=True, blank=True)  # sub tenant

    def __str__(self):
        return f'{self.name}'
