""" Copyright start
  Copyright (C) 2008 - 2023 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end """
"""
Django settings for connector project.

Generated by 'django-admin startproject' using Django 1.8.4.

For more information on this file, see
https://docs.djangoproject.com/en/1.8/topics/settings/

For the full list of settings and their values, see
https://docs.djangoproject.com/en/1.8/ref/settings/
"""

# Import some utility functions
import json
import os
import sys
from pathlib import Path
from concurrent.futures import ThreadPoolExecutor
from os.path import abspath, dirname, join

from utils.config_parser import all_config
from xfutils.logging_guid.consts import LOGGING_HANDLERS, LOGGING_FORMATTERS, LOGGING_FILTERS

ALL_CONFIG = all_config
# ##### PATH CONFIGURATION ################################
# Fetch Django's project directory
DJANGO_ROOT = dirname(dirname(abspath(__file__)))
BASE_DIR = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))

ENABLE_SWAGGER = ALL_CONFIG.get('application', 'enable_swagger')

# The name of the whole site
SITE_NAME = 'integrations'

# Load configurations
CONFIG_DIR = join(DJANGO_ROOT, 'configs')
CONFIG_FILE = 'config.ini'

CRUD_HUB_URL = 'https://' + ALL_CONFIG.get('application', 'app_host')
SECRETS_API = ALL_CONFIG.get('application', 'secrets_api')

# LW_AGENT=False
# AGENT_ID='default'
LW_AGENT = ALL_CONFIG.get('xf.instance.lwAgent', default=False) or os.environ.get('ENV', None) == 'agent'
XF_AGENT = LW_AGENT
AGENT_ID = ALL_CONFIG.get('xf.instance.agentId')
SELF_ID = AGENT_ID
TENANT_ID = os.environ.get('TENANT_ID', SELF_ID)

IS_EXEC_ENV = False
if int(os.environ.get('IS_EXEC_ENV', 0)) == 1:
    IS_EXEC_ENV = True

REGISTER_HTTP_REQUEST_COUNT_METRIC = True

if XF_AGENT or IS_EXEC_ENV:
    REGISTER_HTTP_REQUEST_COUNT_METRIC = False

# RabbitMQ Publisher
CONNECTION_RETRY_COUNT = 3
CONNECTION_RETRY_DELAY = 5
PUBLISH_MESSAGE_RETRY_COUNT = 5

# ##### APPLICATION CONFIGURATION #########################
DJANGO_APPS = [
    'django.contrib.admin',
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.messages',
    'django.contrib.staticfiles',
    'django_filters',
    'rest_framework',
]

THIRD_PARTY_APPS = [
    # 'corsheaders',
]

SHARED_APPS = [
    'django_tenants',
    'tenant',
    'django.contrib.admin',
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.messages',
    'django.contrib.staticfiles',
    'django_filters',
    'rest_framework',
    'django_guid',
    'postman',
    'identity',
    'contenthub',
    'drf_standardized_errors'
]

TENANT_APPS = [
    'taggit',
    'data_import',
    'postman',
    'connectors',
    'replay',
    'agents',
]

# See: https://docs.djangoproject.com/en/dev/ref/settings/#installed-apps
# INSTALLED_APPS = DJANGO_APPS + THIRD_PARTY_APPS + TENANT_APPS

INSTALLED_APPS = list(SHARED_APPS) + [app for app in TENANT_APPS if app not in SHARED_APPS]
TENANT_MODEL = 'tenant.Tenant'
TENANT_DOMAIN_MODEL = 'tenant.Domain'
SHOW_PUBLIC_IF_NO_TENANT_FOUND = True
TENANT_LIMIT_SET_CALLS = True
PUBLIC_SCHEMA_NAME = 'public'

# Middlewares
MIDDLEWARE = [
    'django_guid.middleware.guid_middleware',
    'xfutils.logging_guid.middleware.tenantid_middleware',
    'xfutils.middleware.ProbeMiddleware',
    'tenant.custommiddleware.XHeaderTenantMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware',
]

if REGISTER_HTTP_REQUEST_COUNT_METRIC:
    print("Registering http request count metric")
    MIDDLEWARE.insert(0, 'xfutils.monitor.middleware.get_request_count_middleware')
    # Currently service token auth is only required for http request count metric
    SERVICE_TOKEN_AUTH = {
        'USER': 'XFServiceUser',
        # 0 will be replace by tenantuuid
        'USER_EMAIL': 'XFServiceUser@{0}',
        # Authorization header will be used like 'Service-Token <token>'
        'AUTH_HEADER_TYPE': 'Service-Token',
        'ALGORITHM': 'RS256',
        #
        # ssh-keygen -t rsa -b 2048 -m PEM -f jwtRS256.key
        # openssl rsa -in jwtRS256.key -pubout -outform PEM -out jwtRS256.key.pub
        # awk 'NF {sub(/\r/, ""); printf "%s\\n",$0;}' jwtRS256.key # insert output to SIGNING_KEY
        # awk 'NF {sub(/\r/, ""); printf "%s\\n",$0;}' jwtRS256.key.pub # insert output to VERIFYING_KEY
        #
        # Use 'SIGNING_KEY' for jwt encode.
        # Note that SERVICE_TOKEN_AUTH don't need SIGNING_KEY, hence not added in configuration. It is used by caller.
        # JWT token signed with signing key will be decoded by VERIFYING_KEY.
        #
        'SIGNING_KEY': ALL_CONFIG.get('application', 'SIGNING_KEY').replace('\\n', '\n').strip(),
        'VERIFYING_KEY': ALL_CONFIG.get('application', 'VERIFYING_KEY').replace('\\n', '\n').strip()
    }

# Hosts/domain names that are valid for this site; required if DEBUG is False
# See https://docs.djangoproject.com/en/1.5/ref/settings/#allowed-hosts
ALLOWED_HOSTS = ['*']

# Connector settings
CONNECTORS_DIR = join(DJANGO_ROOT, 'connectors')
CONNECTORS_EXTENSION = '.tgz'
CONNECTOR_DEVELOPMENT_VERSION_POSTFIX = '_dev'
CONNECTORS_RESERVED = ['cyops_utilities', 'smtp']
NOT_ALLOWED_CONN_SELF = ['ssh', 'nmap-scanner']

# ##### DATABASE CONFIGURATION ############################
DB_HOST = ALL_CONFIG.get('database', {}).get('db_host', 'db')
DB_PORT = ALL_CONFIG.get('database', {}).get('db_port', '5432')
DB_NAME = ALL_CONFIG.get('database', 'db_name', 'integration')
DB_USER = ALL_CONFIG.get('database', {}).get('db_user', 'cyberpgsql')
DB_PASSWORD = ALL_CONFIG.get('database', {}).get('db_password', 'changeme')
AUTH_HOST = ALL_CONFIG.get('application', 'auth_host', 'auth')
WF_HOST = ALL_CONFIG.get('application', 'wf_host', 'workflow')

HMAC_TIME_THRESHOLD = 30
HMAC_USER = 'hmac_user'

ENCRYPTION = LW_AGENT
# Decrypt DB password
if DB_PASSWORD and ENCRYPTION:
    from xfutils.passwordModule import manage_password
    output=manage_password(DB_PASSWORD, 'decrypt')
    DB_PASSWORD = output


DATABASES = {
    'default': {
        'ENGINE': 'django_tenants.postgresql_backend',
        'USER': DB_USER,
        'PASSWORD': DB_PASSWORD,
        'NAME': DB_NAME,
        'HOST': DB_HOST,
        'PORT': DB_PORT
    }
}

DATABASE_ROUTERS = (
    'django_tenants.routers.TenantSyncRouter',
)

# ##### SECURITY CONFIGURATION ############################
SECRET_KEY = '1qsjrdu6005n=^ukpx=&_scsrf%28)4xvbo%904-8+@him6$hp'
SEALAB_SECRET_KEY = ALL_CONFIG.get('application', 'SEALAB_SECRET_KEY')

# ##### DJANGO RUNNING CONFIGURATION ######################
# The default WSGI application
WSGI_APPLICATION = '%s.wsgi.application' % SITE_NAME

# The root URL configuration
ROOT_URLCONF = '%s.urls' % SITE_NAME

TEMPLATES = [
    {
        'NAME': 'django',
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'DIRS': [],
        'APP_DIRS': True,
        'OPTIONS': {
            'context_processors': [
                'django.template.context_processors.debug',
                'django.template.context_processors.request',
                'django.contrib.auth.context_processors.auth',
                'django.contrib.messages.context_processors.messages',
            ],
        },
    },
    {
        'NAME': 'jinja2',
        'BACKEND': 'django.template.backends.jinja2.Jinja2',
        'DIRS': [],
        'APP_DIRS': False,
        'OPTIONS': {
            'environment': 'integrations.jinja.environment'
        }
    },
]

REST_FRAMEWORK = {
    'DEFAULT_PAGINATION_CLASS':
        'utils.pagination.HydraLimitOffsetPagination',
    'PAGE_SIZE': 30,
    'DEFAULT_PARSER_CLASSES': (
        'rest_framework.parsers.JSONParser',
        'rest_framework.parsers.MultiPartParser',
    ),
    'DEFAULT_AUTHENTICATION_CLASSES': [
        'utils.schemes.JWTAuthentication',
    ],
    'DEFAULT_PERMISSION_CLASSES': [
        'rest_framework.permissions.IsAuthenticated',
        'utils.permissions.DefaultPermission'
    ],
    'EXCEPTION_HANDLER': 'drf_standardized_errors.handler.exception_handler'
}

CACHE_CONFIG = {
    'prefix': 'xf:integration',
    'version': 'v1',
    'cache_name': 'redis'
}

CACHES = {
    'default': {
        'BACKEND': 'django.core.cache.backends.db.DatabaseCache',
        'LOCATION': 'integrations_cache',
    }
}

if REGISTER_HTTP_REQUEST_COUNT_METRIC:
    # Currently redis cache is only required for http request count metric
    CACHES[CACHE_CONFIG.get('cache_name')] = {
        "BACKEND": "django_redis.cache.RedisCache",
        #
        # db=0 is default database used by Django caching. Explicitly mentioned just not miss out.
        #
        "LOCATION": "redis://{0}:{1}?db=0".format(ALL_CONFIG.get('redis', 'redis_host'),
                                              ALL_CONFIG.get('redis', 'redis_port')),
        "KEY_PREFIX": CACHE_CONFIG.get('prefix'),
        "VERSION": CACHE_CONFIG.get('version'),
        "OPTIONS": {
            "CLIENT_CLASS": "django_redis.client.DefaultClient",
            "PASSWORD": ALL_CONFIG.get('redis', 'redis_password'),
        }
    }

# This site's ID
SITE_ID = 1

# ##### INTERNATIONALIZATION ##############################
LANGUAGE_CODE = 'en-us'

TIME_ZONE = 'UTC'

# Internationalization
USE_I18N = True

# Localization
USE_L10N = True

# enable time zone awareness by default
USE_TZ = True

URL_BASE = 'api/integration'

# Static files (CSS, JavaScript, Images)
# https://docs.djangoproject.com/en/1.9/howto/static-files/
STATIC_URL = '/{0}/static/'.format(URL_BASE)

#
# Ideally, STATIC_ROOT should be somewhere in /opt/integration, as that is base directory.
# But since, in dev environments, we mount code at /opt/integration, static files would not be
# available, as we don't do check-in for it. Hence use STATIC_ROOT which is part of container
# image. i.e /static. During image build, we prepare static contents using below command:
# python manage.py collectstatic --no-input
# Note that when we run Django in DEBUG=True, we don't need to collectstatic, as in DEBUG mode,
# Django makes staticfiles available from app directory. But for production, since DEBUG will be
# false, we need to collect staticfiles using above command.
#
STATIC_ROOT = '/static/'

TMP_FILE_ROOT = '/tmp/'

# DEBUG
DEBUG = False
if ALL_CONFIG.get('application', 'debug'):
    DEBUG = True

# ##### LOGGING ##############################
APP_LOG_FILE_PATH = ALL_CONFIG.get('logging', 'app_log_file_path')
APP_LOG_FILE_SIZE = eval(ALL_CONFIG.get('logging', 'app_log_file_size'))
APP_LOG_FILE_BACKUP_COUNT = int(ALL_CONFIG.get('logging', 'app_log_file_backup_count'))

CONNECTOR_LOGGER_LEVEL = ALL_CONFIG.get('logging', 'connector_logger_level')
DJANGO_LOGGER_LEVEL = ALL_CONFIG.get('logging', 'django_logger_level')

ROUTER_HEARTBEAT = int(ALL_CONFIG.get('router', 'heartbeat', '120'))
ROUTER_CONNECTION_ATTEMPTS = int(ALL_CONFIG.get('router', 'connection_attempts', '5'))
ROUTER_RETRY_DELAY = int(ALL_CONFIG.get('router', 'retry_delay', '10'))
THREAD_POOL_WORKER = int(ALL_CONFIG.get('application', 'THREAD_POOL_WORKER', '16'))

LOGGING = {
    'version': 1,
    'disable_existing_loggers': False,
    'filters': LOGGING_FILTERS,
    'handlers': LOGGING_HANDLERS,
    'formatters': LOGGING_FORMATTERS,
    'loggers': {
        # Below '' is a root logger
        '': {
            'handlers': ['console'],
            'level': ALL_CONFIG.get('logging', 'root_logger_level'),
            'propagate': False
        },
        #
        # Below is special logger which prints which API being accessed by printing API path, correlation_id
        # and tenantuuid.
        #
        'xfutils.logging_guid.middleware': {
            'handlers': ['console'],
            # Don't change this level, as this is special logger which prints which API being accessed.
            'level': 'DEBUG',
            'propagate': False,
        },
        'django.db.backends': {
            'handlers': ['console'],
            'level': CONNECTOR_LOGGER_LEVEL,  # Set the desired log level (DEBUG, INFO, etc.)
        },
        'connectors': {
            'handlers': ['console'],
            'level': CONNECTOR_LOGGER_LEVEL,
            'propagate': False
        },
        'django.request': {
            'handlers': ['console'],
            'level': DJANGO_LOGGER_LEVEL,
            'propagate': False
        },
        'postman': {
            'handlers': ['console'],
            'level': CONNECTOR_LOGGER_LEVEL,
            'propagate': False
        },
        'replay': {
            'handlers': ['console'],
            'level': CONNECTOR_LOGGER_LEVEL,
            'propagate': False
        },
        'pika': {
            'handlers': ['console'],
            'level': CONNECTOR_LOGGER_LEVEL,
            'propagate': False
        },
        'xfutils.monitor': {
            'handlers': ['console'],
            'level': CONNECTOR_LOGGER_LEVEL,  # Set the desired log level (DEBUG, INFO, etc.)
            'propagate': False,
        }
    }
}

if LW_AGENT:
    logpath = '/var/log/xfagent/xfagent.log'
    output_file = Path(logpath)
    output_file.parent.mkdir(exist_ok=True, parents=True)
    output_file.touch(exist_ok=True)
    LOGGING['handlers']['console'] = {
        'class': 'logging.handlers.RotatingFileHandler',
        'filename': logpath,
        'maxBytes': APP_LOG_FILE_SIZE,
        'backupCount': APP_LOG_FILE_BACKUP_COUNT,
        'formatter': 'verbose',
        "filters": ["correlation_id", "tenantid"],
    }

CH_MAX_RETRY_COUNT = 10
CH_RETRY_SLEEP = 10

PRODUCT_YUM_SERVER = os.getenv('product_yum_server', 'repo.fortisoar.fortinet.com')
CONNECTOR_REPO_BASE_URL = 'https://' + PRODUCT_YUM_SERVER + '/xf/solutions/connectors/'
CONNECTORS_JSON = {}
CONNECTOR_TEMPLATES = []
CONNECTORS_REPO_NAMES = []
CONN_RPM_TEMP_DIR = ALL_CONFIG.get('application', 'conn_rpm_temp_dir',
                                   '/opt/cyops-integration/cyops_connector_rpm')

SDK_PORT = '8000'

try:
    with open('/opt/build-info.json') as f:
        build_json = json.loads(f.read())
        RELEASE_VERSION = build_json.get('release-version')
        BUILD_NUMBER = build_json.get('build-number')
except FileNotFoundError:
    RELEASE_VERSION = '1.0.0'
    BUILD_NUMBER = 'dev'
THREAD_POOL_EXECUTOR = ThreadPoolExecutor(max_workers=THREAD_POOL_WORKER)

# Setting it to 1 to ensure main process is free.
# Issue Identified with VT Connector calling on add Config
# To Fetch data from Master
THREAD_POOL_INSTRUCTION_EXECUTOR = ThreadPoolExecutor(max_workers=1)

FSR_NODE = '/api/3/picklists/5de19a34-1d8b-4423-91fc-4c243a5b62fd'
FSR_AGENT = '/api/3/picklists/502cb999-d0fe-466f-9d99-1f3300513e14'
ENABLE_REMOTE_CONNECTOR_OPERATION = ALL_CONFIG.get('application', 'ENABLE_REMOTE_CONNECTOR_OPERATION',
                                                   True)
REPLICATING_LOG = {}
LOG_PAGINATION = 100
MAX_RECORD_LIMIT = '500'
DEFAULT_AUTO_FIELD = 'django.db.models.AutoField'

BLOCK_IN_TEMPLATE = ['__class__', '__base__', '__subclass__', '__builtins__', '__import__', '__globals__', '__init__']

INTEGRATION_DIR = '/opt/integration/'
CONNECTOR_FILES_PATH = 'connector_files'
CONNECTOR_INSTALLATION_DIR = INTEGRATION_DIR + CONNECTOR_FILES_PATH + '/'
DEV_CONNECTOR_INSTALLATION_DIR = '/opt/cyops/configs/integrations/connectors_development/'
USER_INTEGRATIONS = 'fsr-integration'
# HA Information
IS_ACTIVE_PRIMARY = None
NODE_ID = ''

if ENABLE_SWAGGER:
    # By default swagger ask for username password as auth mechanism. Since we need JWT, override it.
    SWAGGER_SETTINGS = {
        #'JSON_EDITOR': True,
        # Disable django admin button on swagger API, since we are using JWT
        'USE_SESSION_AUTH': False,
        'SECURITY_DEFINITIONS': {
            'api_key': {
                'type': 'apiKey',
                'in': 'header',
                'name': 'Authorization'
            }
        },
        'DEFAULT_MODEL_RENDERING': 'example',
        # Hide model ref. https://stackoverflow.com/questions/57138564/how-to-hide-the-models-section-in-swagger-ui
        'DEFAULT_MODEL_DEPTH': -1,
        'DOC_EXPANSION': 'none',
        'TAGS_SORTER': 'alpha',
        'DISPLAY_OPERATION_ID': False
    }
    # To server static files required by swagger
    MIDDLEWARE.append('whitenoise.middleware.WhiteNoiseMiddleware')
    STATICFILES_STORAGE = 'whitenoise.storage.CompressedManifestStaticFilesStorage'
    INSTALLED_APPS.append('drf_yasg')