""" Copyright start
  Copyright (C) 2008 - 2023 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end """
from django.urls import include, path, re_path
from rest_framework import routers
from django.conf import settings

from connectors.views import ConnectorConfigurationView, ConnectorOperationView, ConnectorHealth
from data_import.views import DataImportView

# Routers provide an easy way of automatically determining the URL conf.
router = routers.DefaultRouter()
router.register(r'configuration', ConnectorConfigurationView, basename='configuration')
router.register(r'operation', ConnectorOperationView, basename='operation')
router.register(r'data-import', DataImportView, basename='data import')
router.register(r'healthcheck', ConnectorHealth, basename='check health')

urlpatterns = [
    path('api/integration/', include('connectors.urls')),
    path('api/integration/', include(router.urls)),
    path('api/integration/', include('tenant.urls')),
    path('api/integration/', include('agents.urls')),
    path('api/integration/', include('contenthub.urls')),
    path('api/integration/', include('porter.urls')),
    path('api/integration/query/', include('query.urls'))
]

if settings.ENABLE_SWAGGER:
    # Credit https://www.jasonmars.org/2020/04/22/add-swagger-to-django-rest-api-quickly-4-mins-without-hiccups/
    # drf_yasg code starts here
    from rest_framework import permissions
    from drf_yasg.views import get_schema_view
    from drf_yasg import openapi

    schema_view = get_schema_view(
        openapi.Info(
            title="Integration Service API Documentation",
            default_version='v1',
        ),
        public=True,
        authentication_classes=[],
        permission_classes=(permissions.AllowAny,),
    )
    # ends here

    #
    # Endpoint for download OpenAPI 2.0 spec in json/yaml format.
    # We use drf-yasg package which only support OpenAPI 2.0 spec.
    # For now, commenting download spec API. We can enable when required.
    #
    #urlpatterns.append(re_path(r'^{0}/docs/swagger(?P<format>\.json|\.yaml)$'.format(settings.URL_BASE), schema_view.without_ui(cache_timeout=0), name='schema-json'))
    urlpatterns.append(re_path(r'{0}/docs/'.format(settings.URL_BASE), schema_view.with_ui('swagger', cache_timeout=0), name='schema-swagger-ui'),)
