#!/bin/bash
CURRENT_DIR=$(dirname "$0")
CURRENT_DIR=$(cd "$CURRENT_DIR" && pwd)
DJANGO_APP_CONTAINER="./"

MANAGE_PY="$DJANGO_APP_CONTAINER/manage.py"

# copy .pth file to python environment
cp -nr $CURRENT_DIR/connector_packages.pth /usr/local/lib/python3.11/site-packages/

manage_db_password() {
    printf "database:\n  db_password: $(python $MANAGE_PY manage_password encrypt "$DBPW")" > /opt/integration/configs/agent/db_config.yml
    unset DBPW
}

db_migrate() {
    #python $MANAGE_PY makemigrations
    python $MANAGE_PY migrate

    if [ "$ENV" = "agent" ]; then
        python $MANAGE_PY create_xf_tenant --tenant_id $TENANT_ID --schema_name $SCHEMA --name $TENANT_ID --active True
        python $MANAGE_PY create_self_agent_or_update_version --schema_name $SCHEMA
    else
        # Todo for dev repo do the etc host entry
        python $MANAGE_PY sync
    fi
}

install_connectors_from_file() {
    # We use & here to run this in the background so that this command will start to run right before we start gunicorn
    python $MANAGE_PY install_agent_connectors --schema_name $SCHEMA --filepath /opt/integration/connectors/agent_connectors.json --max_retries 60 &
}


if [[ ! -z "${DBPW}" ]] && [[ ! -f "/opt/integration/configs/agent/db_config.yml" ]]; then
    manage_db_password
fi

# Check if migration is required and perform it on migration job and exit.
if [ "$migrate" = "true" ]; then
    db_migrate
    exit "$?"
fi

# Perform migration if the environment is 'agent'
if [ "$ENV" = "agent" ]; then
    db_migrate
fi

#BUILD_ENV=local_dev
db_migrate

#
# Need to create required site-packages dirs, else after executing connector action, getting
# errors like below:
# "Error message": "No module named 'vobject'  Connector :: exchangeV4.2.3
#
mkdir -p /opt/cyops/configs/integrations/packages/bin/
mkdir -p /opt/cyops/configs/integrations/packages/lib/python3.11/site-packages/

if [ "$ENV" = "agent" ]; then
    install_connectors_from_file
fi
#
# exec is required to recieve SIGTERM signal on container stop.
# Gunicorn will gracefully shutdown workers on SIGTERM
#
exec gunicorn -c $CURRENT_DIR/gunicorn.conf.py --reload --keep-alive 0