""" Copyright start
  Copyright (C) 2008 - 2023 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end """
import datetime
import errno
import os
import json
from utils.config_parser import all_config
from xfutils.utils import normal_round, GunicornCustomLogger, generate_jwt_token, api_call
import subprocess
from sys import executable, exit

logger_class = GunicornCustomLogger
# Django WSGI application path in pattern MODULE_NAME:VARIABLE_NAME
wsgi_app = "integrations.wsgi:application"
# The granularity of Error log outputs
loglevel = all_config.get('gunicorn', 'loglevel')
# The number of worker processes for handling requests
workers = normal_round(2 * float(all_config.get('container', 'cpu'))) + 1
print("Worker count is %s" % workers)
# The socket to bind
bind = ':{0}'.format(all_config.get('gunicorn', 'port'))
#TODO check with ben niraj
CURRENT_ENV = os.environ.get('ENV')
if CURRENT_ENV and CURRENT_ENV == 'agent':
  # If agent, then log to file, else goes to console
  accesslog = errorlog = "/var/log/gunicorn.log"
# Redirect stdout/stderr to log file
capture_output = True
# Decrease the time for stopping the workers (Time in sec)
graceful_timeout = all_config.get('gunicorn', 'graceful_timeout')
timeout = all_config.get('gunicorn', 'timeout')

# Daemonize the Gunicorn process (detach & enter background)
# daemon = True
certfile = "/opt/integration/package.d/server-cert-chained.pem"
keyfile = "/opt/integration/package.d/server-key.pem"
# Any value greater than zero(default is 0) will limit the number of requests a worker will process
# before automatically restarting. This is a simple method to help limit the damage of memory leaks.
# We get below logs for this.
# [24] [INFO] Autorestarting worker after current request.
# [24] [INFO] Worker exiting (pid: 24)
# [28] [INFO] Booting worker with pid: 28
#
max_requests = all_config.get('gunicorn', 'max_requests')
#
# The jitter causes the restart per worker to be randomized by randint(0, max_requests_jitter).
# This is intended to stagger worker restarts to avoid all workers restarting at the same time.
#
max_requests_jitter = all_config.get('gunicorn', 'max_requests_jitter')

reload_extra_files = ['/opt/xf/configs/.config_changed']

timeout = 600

def on_starting(worker):
    if CURRENT_ENV != 'agent':
        return
    agent_id = all_config.get('xf.instance.agentId')
    tenant_id = os.environ.get('TENANT_ID')

    with open('/opt/build-info.json') as f:
        build_json = json.loads(f.read())
        RELEASE_VERSION = build_json.get('release-version')
        BUILD_NUMBER = build_json.get('build-number')

    payload = {
      'agent': all_config.get('xf.instance.agentId'),
      'healthStatus': {
          'status': 'connected',
          'message': 'Agent has been connected'
      },
      'version': RELEASE_VERSION,
      'timestamp': datetime.datetime.now().strftime('%Y-%m-%dT%H:%M:%S.%fZ'),
      'request_data': {
          'tenantid': tenant_id,
          'username': agent_id,
          'useruuid': all_config.get('xf.instance.applianceUuid')
      }
    }
    
    print("Checking Agent connectivity...")
    connectivity_check_cmd = [executable, '/opt/integration/manage.py', 'agent_connectivity_check']

    # call the conectivity check command. This  will fail if the db connection isn't present,
    # and will throw an exception if the rmq connection cannot be made. If this succeeds, log
    # the output and try to send a health status message to XF. If it fails, send a status failed
    # message to XF and exit
    try:
        process = subprocess.run(connectivity_check_cmd, check=True, capture_output=True, encoding='utf-8')
        for line in process.stdout.splitlines():
            print(line)
    except subprocess.CalledProcessError as e:
        for line in e.stdout.splitlines():
            print(line)
        print(e.stderr)
        payload['healthStatus'] = {
            "status": "disconnected",
            "message": str(e)
        }
        _send_agent_health_update_request(payload, tenant_id)
        exit(errno.EINTR)
    _send_agent_health_update_request(payload, tenant_id)

def on_exit(server):
    if CURRENT_ENV != 'agent':
        return
    agent_id = all_config.get('xf.instance.agentId')
    tenant_id = os.environ.get('TENANT_ID')
    payload = {
            'agent': all_config.get('xf.instance.agentId'),
            'healthStatus': {
                'status': 'disconnected',
                'message': 'Agent has been shut down'
            },
            'timestamp': datetime.datetime.now().strftime('%Y-%m-%dT%H:%M:%S.%fZ'),
            'request_data': {
              'tenantid': tenant_id,
              'username': agent_id,
              'useruuid': all_config.get('xf.instance.applianceUuid')
          }
        }
    _send_agent_health_update_request(payload, tenant_id)


def _send_agent_health_update_request(data, tenant_id):
    url = f"https://{all_config.get('application', 'integration_host')}/api/integration/connector-ack/agent-healthcheck/"
    token = generate_jwt_token(all_config.get('xf.instance.applianceUuid'),
                               all_config.get('xf.instance.applianceKey'), tenantid=tenant_id,
                               type='appliance')
    try:
        # @remove if not needed
        token = token.decode('utf-8')
    except:
        pass
    headers = {
        "Authorization": f"Bearer {token}"
    }
    if not data.get('token'):
        data['token'] = token
    print(f"Sending Agent health status update: {data['healthStatus']['status']}")
    ack_response = api_call(url, method='POST', body=data, headers=headers, verify=False)