""" Copyright start
  Copyright (C) 2008 - 2023 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end """
import sys, os
import signal
from threading import Thread
from django_tenants.utils import schema_context

from django.apps import AppConfig
from django.conf import settings

from connectors.core.connector import logger
from utils.globals import initialize as initialize_globals
from utils.shutdown import sigabrt_handler, sigterm_handler


class PostmanConfig(AppConfig):
    name = 'postman'

    def ready(self):
        # start for tenant pods
        if 'makemigrations' in sys.argv or 'migrate' in sys.argv or 'clearcache' in sys.argv:
            return
        if check_for_exit_command():
            return
        if settings.XF_AGENT:
            with schema_context(os.environ.get('SCHEMA')):
                create_agent_or_update_version()
                start_agent()


def check_for_exit_command():
    exit_commands = ['setupsdk.py', 'execute_action', 'import_connectors_from_path', 'manage.py']
    for argv in sys.argv:
        for exit_command in exit_commands:
            if exit_command in argv:
                return True


def start_agent():
    from postman.core.consumer import start_consumer
    initialize_globals()
    signal.signal(signal.SIGTERM, sigterm_handler)
    signal.signal(signal.SIGABRT, sigabrt_handler)
    logger.info('Spawning Consumer thread')
    t = Thread(target=start_consumer, name='Agent Consumer', args=(settings.SELF_ID))
    #start_consumer(settings.SELF_ID)
    t.start()

def create_agent_or_update_version():
    from agents.models import Agent
    agent, created = Agent.objects.get_or_create(agentId=settings.SELF_ID, defaults={'name': 'Self Agent', 'isLocal': True},
        applianceKey=settings.ALL_CONFIG.get('xf.instance.applianceKey'),
        applianceUuid=settings.ALL_CONFIG.get('xf.instance.applianceUuid'),
        version=settings.RELEASE_VERSION)
    if not created and agent.version != settings.RELEASE_VERSION:
        agent.version = settings.RELEASE_VERSION
        agent.save()