""" Copyright start
  Copyright (C) 2008 - 2023 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end """
import pika
from django.conf import settings
from django_guid import get_guid
from connectors.core.connector import logger
from postman.core.exceptions import MessagePublishFailed
from postman.utils.rmq_utility import get_rmq_connection, prepare_intra_cyops_config, prepare_agent_cyops_config


class Publisher:
    def __init__(self, config, connection=None):
        self.config = config  # This is publisher config- it has exchangename routing key to publish
        self.connection = connection
        self.created_new_connection = False

    def _create_connection(self, tenant_id):
        config = prepare_agent_cyops_config(tenant_id)
        conn = get_rmq_connection(**{'config': config, 'publisher': True})
        return conn

    def publish(self, message, tenant_id=None, content_type=None, delivery_mode=2, *args, **kwargs):
        callback_name = self.config.get('callback_name')

        logger.info("Publishing to tenant: " + str(tenant_id))

        try:
            # raise Exception('message')
            if not self.connection and tenant_id:
                logger.info('Creating new connection to publish')
                self.connection = self._create_connection(tenant_id)
                self.created_new_connection = True

            channel = self.connection.channel()
            properties_dict = {
                'delivery_mode': delivery_mode
            }
            correlation_id = kwargs.get('correlation_id')
            callback_queue = kwargs.get('callback_queue')
            if content_type:
                properties_dict['content_type'] = content_type
            if correlation_id:
                properties_dict['correlation_id'] = correlation_id
            if callback_queue:
                properties_dict['reply_to'] = callback_queue
            headers = {
                #
                # Setting tenant_id value to the tenantuuid. Logging framework expect tenantuuid.
                # Since only tenant_id is available here, setting it. Note that there is no harm
                # in doing this. Consumer will show tenantid instead of tenantuuid in logger.
                #
                'tenantuuid':  tenant_id,
                'correlation_id': get_guid()
            }
            properties_dict['headers'] = headers
            properties = pika.BasicProperties(**properties_dict)
            channel.basic_publish(exchange=self.config['exchangeName'],
                                  routing_key=self.config['routingKey'],
                                  body=message,
                                  properties=properties
                                  )
            logger.info("Published message Successfully: " + str(self.config))
            channel.close()
        except Exception as e:
            logger.exception('Publish failed to remote due to connectivity issue with message exchange: %s', str(e))
            raise MessagePublishFailed(repr(e))
        finally:
            if self.created_new_connection:
                self.connection.close()


if __name__ == '__main__':
    pass
