""" Copyright start
  Copyright (C) 2008 - 2023 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end """
import ssl
import pika
from django.conf import settings
from pika.credentials import ExternalCredentials
from connectors.core.connector import logger
from xfutils.passwordModule import manage_password
from postman.utils.constants import *

all_config = settings.ALL_CONFIG


def create_exchange(config, agent_id='', *args, **kwargs):
    from connectors.environment import expand
    connection = get_rmq_connection(**{'config': config, 'consumer': True})
    for exchange in config.get('rabbit').get('exchange', []):
        exchange = {key: expand({'self_id': agent_id}, value) for key, value in exchange.items()}
        config.update(exchange)
        create_rmq_exchange(config, connection)
    if connection:
        connection.close()


def create_rmq_exchange(config, connection, *args, **kwargs):
    exchange_options = config['exchangeOptions']
    channel = connection.channel()
    channel.exchange_declare(exchange=config['exchangeName'],
                             exchange_type=config['exchangeType'],
                             passive=exchange_options['passive'],
                             durable=exchange_options['durable'],
                             auto_delete=exchange_options['autoDelete'],
                             internal=exchange_options['internal'])
    channel.close()


def get_rmq_connection(config, consumer=False, *args, **kwargs):
    try:
        credentials = ExternalCredentials()
        logger.debug(f"CONFIG IS {config}")

        heartbeat = config.get('heartbeat', settings.ROUTER_HEARTBEAT)
        retry_delay = settings.ROUTER_RETRY_DELAY if kwargs.get('consumer') else 10
        connection_attempts = settings.ROUTER_CONNECTION_ATTEMPTS if kwargs.get('consumer') else 2
        if config['ssl']:
            context = ssl.create_default_context(cafile=config.get('server_cert_file'))
            context.load_cert_chain(config.get('client_cert_file'), config.get('client_pvtkey_file'))
            ssl_options = pika.SSLOptions(context, server_hostname=config.get('sni'))
            parameters = pika.ConnectionParameters(config['host'],
                                                   config['port'],
                                                   config['virtualHost'],
                                                   credentials,
                                                   heartbeat=heartbeat,
                                                   retry_delay=retry_delay,
                                                   connection_attempts=connection_attempts,
                                                   ssl_options=ssl_options)

        else:
            parameters = pika.ConnectionParameters(config['host'],
                                                   config['port'],
                                                   config['virtualHost'],
                                                   credentials,
                                                   heartbeat=heartbeat,
                                                   retry_delay=settings.ROUTER_RETRY_DELAY,
                                                   connection_attempts=settings.ROUTER_CONNECTION_ATTEMPTS,
                                                   )
        connection = pika.BlockingConnection(parameters)
        if consumer:
            update_agent_status(config.get('id'), 'success')
    except Exception as e:
        logger.exception('message')
        if config.get('consumer'): update_agent_status(config.get('id'), 'failed')
        raise e
    return connection


def update_agent_status(agent_id, status='success', *args, **kwargs):
    pass
    #Agent.objects.filter(agentId=agent_id).update(status=status)

def prepare_intra_cyops_config(agent_id, *args, **kwargs):
    config = dict()
    logger.error("Preparing intra cyops config: " + str(all_config.get('mq')))
    config['userName'] = all_config.get('rmq_public', {}).get('user')
    config['password'] = manage_password(all_config.get('rmq_public', {}).get('password'), 'decrypt', False)
    config['host'] = all_config.get('rmq_public', {}).get('host')
    config['port'] = all_config.get('rmq_public', {}).get('port')
    config['virtualHost'] = all_config.get('rmq_public', {}).get('vhost')
    config['ssl'] = False
    config['id'] = agent_id
    config['role'] = 'self'
    config['rabbit'] = all_config.get('rabbitmq_cyops')
    return config


def prepare_agent_cyops_config(agent_id):
    from connectors.environment import expand
    from agents.models import Agent
    agent_obj = Agent.objects.filter(agentId=agent_id).first()
    
    config = dict()
    config['host'] = all_config.get('rmq_public', {}).get('host')
    config['sni'] = all_config.get('rmq_public', {}).get('sni') if all_config.get('rmq_public', {}).get('sni') else config['host']
    config['port'] = all_config.get('rmq_public', {}).get('port')
    config['ssl'] = all_config.get('rmq_public', {}).get('ssl')
    config['server_cert_file'] = all_config.get('rmq_public', {}).get('caChainBundle', all_config.get('rmq_public', {}).get('caCertPath'))
    config['virtualHost'] = agent_id
    config['id'] = agent_id
    config['rabbit'] = all_config.get('rabbitmq_cyops')

    if settings.LW_AGENT:
        config['client_cert_file'] = all_config.get('rmq_public', {}).get('clientCertFileName')
        config['client_pvtkey_file'] = all_config.get('rmq_public', {}).get('clientKeyFileName')
    else:
        config['client_cert_file'] = expand({'agent_id': agent_id}, all_config.get('rmq_public', {}).get('clientCertFileName'))
        config['client_pvtkey_file'] = expand({'agent_id': agent_id}, all_config.get('rmq_public', {}).get('clientKeyFileName'))
        with open(config['client_cert_file'], 'w') as f:
            f.write(agent_obj.clientCertificate)
            f.close()
        with open(config['client_pvtkey_file'], 'w') as f:
            f.write(agent_obj.clientKey)
            f.close()

    return config
