import logging

import jwt
from django_tenants.middleware.main import TenantMainMiddleware

from django.conf import settings

logger = logging.getLogger(__name__)
from tenant.models import Tenant
import glob

SKIP_PATHS = [
    '/api/integration/docs',
    '/api/integration/static',
]


class XHeaderTenantMiddleware(TenantMainMiddleware):
    """
    Determines tenant by the value of the ``X-DTS-SCHEMA`` HTTP header.
    """

    def __init__(self, get_response):
        super().__init__(get_response)
        self.decode_token = {}

    def no_tenant_found(self, request, hostname):
        if any(path in request.path for path in SKIP_PATHS):
            return
        super().no_tenant_found(request, hostname)

    def hostname_from_request(self, request):
        if any(path in request.path for path in SKIP_PATHS):
            # This is required in schemes.py in auth middleware
            request.public = True
            return

        # This is sent through workflow
        # Inter pod communication is unauth
        # So token might be not provided
        secret_key = request.GET.get('secretKey')
        if secret_key and secret_key == settings.SEALAB_SECRET_KEY:
            tenant_id = request.headers.get('TenantId')
            return tenant_id

        token = request.headers.get('Authorization')
        token = token[7:]
        decode_token = jwt.decode(token, algorithms=['HS256'], options={'verify_signature': False})
        self.decode_token = decode_token
        return decode_token.get('tenantid')

    def get_tenant(self, domain_model, hostname):
        if hostname:
            try:
                tenant = Tenant.objects.get(tenant_id=hostname)
                return Tenant.objects.get(tenant_id=hostname)
            except Exception as e:
                logger.error('Error while getting tenant {0} ERROR: {1}'.format(hostname, str(e)))
        raise domain_model.DoesNotExist()
        # try:
        #     # hostname value is set as tenantid in hostname_from_request
        #     tenant = Tenant.objects.get(tenantid=hostname)
        #     return tenant
        # except Exception as e:
        #     logger.error('Error while getting tenant {0} ERROR: {1}'.format(hostname, str(e)))
        #     raise domain_model.DoesNotExist()
