import logging

from rest_framework import status
from rest_framework import viewsets
from rest_framework.decorators import action
from rest_framework.response import Response
from tenant.models import Tenant, Domain
from tenant.serializer import TenantSerializer, DomainSerializer
from tenant.utils import load_fixtures, install_system_connectors, create_self_agent

logger = logging.getLogger(__name__)


class TenantViewSet(viewsets.ModelViewSet):
    """
    A viewset for viewing and editing tenant instances.
    """
    queryset = Tenant.objects.all()
    serializer_class = TenantSerializer

    @action(methods=['POST'], detail=False)
    def provision(self, request):
        tenantid = request.tenantid
        if not isinstance(tenantid, str):
            tenantid = str(tenantid)
        filter_tenant_id = tenantid.translate({ord(c): "" for c in "!@#$%^&*()[]{};:,./<>?\|`~-=_+"})
        schema_name = "schema" + filter_tenant_id

        try:
            tenant, created = Tenant.objects.get_or_create(tenant_id=tenantid, name=tenantid, schema_name=schema_name,
                                                           uuid=request.tenantuuid, active=True)
        except Exception as e:
            error_message = "Error while creating or getting tenant with id: {0}".format(tenantid)
            logger.error("{0} ERROR: {1}".format(error_message, str(e)))
            return Response({'message': error_message}, status=status.HTTP_400_BAD_REQUEST)
        create_self_agent(schema_name, tenantid)

        try:
            load_fixtures(schema_name)
            install_system_connectors(schema_name, tenantid)
            return Response({'tenantid': tenantid})
        except Exception as e:
            return Response({'message': str(e)}, status=status.HTTP_400_BAD_REQUEST)


class DomainViewSet(viewsets.ModelViewSet):
    """
    A viewset for viewing and editing Domain instances.
    """
    queryset = Domain.objects.all()
    serializer_class = DomainSerializer
