######
# IMP: This file getting imported in gunicorn config file. Do not import any Django
# related file in this.
#####
import os
from os.path import abspath, dirname, join

from xfutils.config_parser import ConfigReader

# ENV can be dev/qa/prod
CURRENT_ENV = os.environ.get('ENV', 'dev')
if not CURRENT_ENV:
    raise ValueError("Undefined environment variable 'ENV'")

DEFAULT_CONFIG_DIR = join(dirname(dirname(abspath(__file__))), 'configs', 'default')
CONFIG_DIR = join(dirname(dirname(abspath(__file__))), 'configs', CURRENT_ENV)

CONFIG_FILES_PROPERTIES = []
CONFIG_FILES_INI = [
    join(DEFAULT_CONFIG_DIR, 'config.ini'),
    join(CONFIG_DIR, 'config.ini')
]
CONFIG_FILES_YAML = [
    join(DEFAULT_CONFIG_DIR, 'config.yml'),
    join(DEFAULT_CONFIG_DIR, 'agent_config.yml'),
    join(DEFAULT_CONFIG_DIR, 'db_config.yml'),
    join(CONFIG_DIR, 'config.yml'),
    join(CONFIG_DIR, 'agent_config.yml'),
    join(CONFIG_DIR, 'db_config.yml')
]

#
# First read default, and then env specific config
#
CONFIG_FILES_K8s = ['/opt/xf/configs/integration_config.yaml', '/opt/xf/configs/resource_config.yaml']
CONFIG_FILES_YAML.extend(CONFIG_FILES_K8s)
config_reader_kwargs = {
    'default_config_dir': DEFAULT_CONFIG_DIR,
    'config_dir': CONFIG_DIR,
    'CONFIG_FILES_PROPERTIES': CONFIG_FILES_PROPERTIES,
    'CONFIG_FILES_INI': CONFIG_FILES_INI,
    'CONFIG_FILES_YAML': CONFIG_FILES_YAML,
}
#
# ConfigReader is singleton
# Order of reading the files is first properties, then ini and then yaml.
# yaml files are read last since K8s expose configs in yaml format.
#
all_config = ConfigReader(**config_reader_kwargs)