import logging
import sys
import time
import threading
from utils import globals

logger = logging.getLogger(__name__)

#
# Need to use threading.Event as each worker process will recieve SIGTERM and below event will be set.
# Each worker process run single thread running pika consumer. Pika consumer refer to below event
# before processing task. If we use normal global variable, then thread don't see updated value,
# and it keep processing, hence need to threading event.
#
SIGTERM_RECEIVED = threading.Event()
SIGABRT_RECEIVED = threading.Event()

#
# Each gunicorn process just fork one thread and starts pika consumer in it.
# Hence using global variable without thread safety is good.
# active_tasks getting used in thread, but only single thread is there, so no issue.
#
active_tasks = 0

def sigterm_handler(signum, frame):
    logger.warning("Received SIGTERM, initiating graceful shutdown")
    global SIGTERM_RECEIVED
    SIGTERM_RECEIVED.set()
    global active_tasks
    #
    # Please note that below loop is NOT infinite, even if active_tasks never becomes to 0 because of any
    # reason. This is because, Kubernetes SIGTERM invoked this code, and if pod doesn't gracefully
    # shutdown within configured time(30 secs default), then Kubernetes will forcefully kill
    # pod.
    #
    while active_tasks != 0:
        logger.warning(f"Sleeping for 1 sec, current active tasks count is {active_tasks}")
        time.sleep(1)

    if globals.consumer:
        globals.consumer.connection.add_callback_threadsafe(globals.consumer.stop_consuming)

    sys.exit(0)

def sigabrt_handler(signum, frame):
    logger.warning("Received SIGABRT, initiating graceful shutdown")
    global SIGABRT_RECEIVED
    SIGABRT_RECEIVED.set()
    global active_tasks
    logger.warning(f'ACTIVE TASKS IS {active_tasks}')
    #
    # Please note that below loop is NOT infinite, even if active_tasks never becomes to 0 because of any
    # reason. This is because, Kubernetes SIGTERM invoked this code, and if pod doesn't gracefully
    # shutdown within configured time(30 secs default), then Kubernetes will forcefully kill
    # pod.
    #
    while active_tasks != 0:
        logger.warning(f"Sleeping for 1 sec, current active tasks count is {active_tasks}")
        time.sleep(1)

    if globals.consumer:
        globals.consumer.connection.add_callback_threadsafe(globals.consumer.stop_consuming)

    sys.exit(0)

def track_task(func):
    def wrapper(*args, **kwargs):
        global active_tasks
        active_tasks += 1
        try:
            return func(*args, **kwargs)
        finally:
            active_tasks -= 1
    return wrapper
