Name: xf-integration-agent
Prefix: /opt/integration
Summary: XFabric Agent Package
Version: %{_version}
Release: %{build_number}
License : Commercial
Vendor : Fortinet
Packager: devops-xfabric@fortinet.com
URL: https://www.fortinet.com
AutoProv: no
AutoReq: no
Source: %{name}-%{version}-%{build_number}.tar.gz

%define pgver 14
%define xf_username xfabric
%define xf_group xfabric
%define venv_name .env
%define venv_cmd /usr/bin/python3.11 -m venv
%define venv_dir %{buildroot}%{prefix}/%{venv_name}


# Build deps
BuildRequires: python3-libselinux, postgresql%{pgver}-devel, libffi-devel, python3-libxml2, libxslt-devel, libxml2, liblognorm, openssl-devel, python3.11, python3.11-devel

# Installation deps
Requires: lsof python3.11-devel, python3.11

%description
XFabric Agent Package

%prep
%setup -q -n %{name}-%{version}-%{build_number}

%install
rm -rf %{buildroot}/*
mkdir -p %{buildroot}%{prefix}

rm Dockerfile*
rm -rf integration/configs/dev
rm -rf integration/configs/prod
rm -rf integration/configs/qa
rm -rf deployment/*

cp -r ./* %{buildroot}/opt/
%{venv_cmd} %{venv_dir}

#required to build pyscopg2 from source

%{venv_dir}/bin/python3 -m pip install cython
%{venv_dir}/bin/python3 integration/cythonize.py build_ext --inplace
rm -f integration/cythonize.py
export PATH=$PATH:/usr/pgsql-%{pgver}/bin/
%{venv_dir}/bin/pip install -r %{buildroot}%{prefix}/package.d/requirements.txt

venvctrl-relocate --source=%{venv_dir} --destination=%{prefix}/%{venv_name}
sed -i "s#%{venv_dir}#%{prefix}/%{venv_name}#g" %{venv_dir}/pyvenv.cfg
exit 0

%pre
mkdir -p %{install_log_dir}
{
    set -x
    echo "Date: `date`"
    echo "%{name}: pre"
    echo "============================================"

    # add the xf user and group
    getent group %{xf_group} >/dev/null
    if [ $? -ne 0 ];then
        /usr/sbin/groupadd --force \
                           %{xf_group}
    fi
    getent passwd %{xf_username} >/dev/null
    if [ $? -ne 0 ];then
        # Create xf user
        /usr/sbin/useradd --gid %{xf_group} \
                          --comment "xf" \
                          %{xf_username}

        echo "Cmnd_Alias INSTALL = /usr/bin/yum, /bin/yum, /bin/rpm, /usr/bin/rpm, /bin/sh" >> /etc/sudoers
        echo "%{xf_username}       ALL=(ALL)       NOPASSWD: INSTALL" >> /etc/sudoers
    fi

    echo "============================================"
} >> %{install_log_dir}/%{install_log_file} 2>&1
# %pre ends here

%post
mkdir -p %{install_log_dir}
{
    set -x
    echo "Date: `date`"
    echo "%{name}: post"
    echo "============================================"

    if [ $1 = 1 ]; then
        # Write post install actions here
        echo ""
    fi

    # lsof path broken for connectors https://bugzilla.redhat.com/show_bug.cgi?id=1260300
    ln -sf /usr/bin/lsof /usr/sbin/lsof

} >> %{install_log_dir}/%{install_log_file} 2>&1
# %post ends here


%files
%dir /opt/
%dir /opt/integration
%dir /opt/integration/.env
/opt/integration/*
/opt/rpm/*
/opt/integration/.env/*